;;; php-mode.el --- major mode for editing PHP code

;; Copyright (C) 1999-2003 Turadg Aleahmad

;; Maintainer: Turadg Aleahmad <turadg at users.sourceforge.net>
;; Keywords: php languages oop
;; Created: 1999-05-17
;; Modified: 2003-09-28
;; X-URL:   http://php-mode.sourceforge.net/

(defconst php-version "1.0.5"
  "PHP Mode version number.")

;;; License

;; This program is free software; you can redistribute it and/or
;; modify it under the terms of the GNU General Public License
;; as published by the Free Software Foundation; either version 2
;; of the License, or (at your option) any later version.

;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with this program; if not, write to the Free Software
;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

;;; Usage

;; Rename this file to php-mode.el if it isn't already then place it in
;; your Emacs lisp path (eg. site-lisp) and add to your .emacs file:
;;   (require 'php-mode)

;; If you want colorization, turn on global-font-lock or
;; add this to your .emacs:
;;   (add-hook 'php-mode-user-hook 'turn-on-font-lock)

;; To use abbrev-mode, add lines like this:
;;   (add-hook 'php-mode-user-hook
;;     '(lambda () (define-abbrev php-mode-abbrev-table "ex" "extends")))

;; To make php-mode compatible with html-mode, see http://php-mode.sf.net

;; Many options available under Help:Customize
;; Options specific to php-mode are in
;;  Programming/Languages/Php
;; Since it inherits much functionality from c-mode, look there too
;;  Programming/Languages/C

;;; Commentary:

;; PHP mode is a major mode for editing PHP 3 and 4 source code.  It's
;; an extension of C mode; thus it inherits all C mode's navigation
;; functionality.  But it colors according to the PHP grammar and indents
;; according to the PEAR coding guidelines.  It also includes a couple
;; handy IDE-type features such as documentation search and a source
;; and class browser.


;;; Contributors: (in chronological order)

;; Juanjo, Torsten Martinsen, Vinai Kopp, Sean Champ, Doug Marcey,
;; Kevin Blake, Rex McMaster, Mathias Meyer, Boris Folgmann, Roland
;; Rosenfeld, Fred Yankowski, Craig Andrews, John Keller, Ryan
;; Sammartino, ppercot, Valentin Funk, Stig Bakken, Gregory Stark,
;; Chris Morris

;; 1.05
;;   Incorporated speedbar defs by Gerrit Riessen
;;   Add "foreach" to conditional introducing keywords (Nils Rennebarth)
;;   Cleared the Changelog
;;   Moved contribution credits into comments above


;;; Code:
(require 'speedbar)
(require 'font-lock)
(require 'regexp-opt)
(require 'cc-mode)
(require 'custom)
(require 'apropos)
(require 'cl)
(require 'c-comment-edit)

;; Local variables
(defgroup php nil
  "Major mode for editing PHP code."
  :prefix "php-"
  :group 'languages)

(defcustom php-speedbar-config t
  "*When set to true automatically configures Speedbar to observe PHP files.\
Ignores php-file patterns option; fixed to expression \"\\.\\(p?inc\\|php[s34]?\\|phtml?\\|p[34]\\)\""
  :type 'boolean
  :group 'php)

(defcustom php-mode-speedbar-open nil
  "Normally php-mode starts with the speedbar closed.\
Turning this on will open it whenever php-mode is loaded."
  :type 'boolean
  :group 'php)

(defcustom php-manual-url "http://www.php.net/manual/ja/manual.php"
  "*URL at which to find PHP manual.\
You can replace \"en\" with your ISO language code."
  :type 'string
  :group 'php)

(defcustom php-search-url "http://www.php.net/"
  "*URL at which to search for documentation on a word"
  :type 'string
  :group 'php)

;;;###autoload
(defcustom php-file-patterns (list "\\.php[s34]?\\'" "\\.phtml\\'" "\\.p?inc\\'" "\\.p[34]\\'")
  "*List of file patterns for which to automatically invoke php-mode."
  :type '(repeat (regexp :tag "Pattern"))
  :group 'php)

(defcustom php-mode-user-hook nil
  "List of functions to be executed on entry to php-mode"
  :type 'hook
  :group 'php)

(defcustom php-mode-force-pear nil
  "Normally PEAR coding rules are enforced only when the filename contains \"PEAR\"\
Turning this on will force PEAR rules on all PHP files."
  :type 'boolean
  :group 'php)

;; Note whether we're in XEmacs
(defconst xemacsp (string-match "Lucid\\|XEmacs" emacs-version)
  "Non nil if using XEmacs.")

;;; Typing shortcuts
(define-skeleton php-insert-comment
  "Inserts a full comment." nil
  "/* " _ " */")

;; Define function name completion function
;;(defun php-complete-function ()
;;  "Complete the function name at the point from known PHP functions."
;;  (interactive)
;;  (message "php-complete-function not implemented yet")
;;  ;; how to read the list of functions from a separate file?
;;  )


;;;###autoload
(define-derived-mode php-mode c-mode "PHP"
  "Major mode for editing PHP code.\n\n\\{php-mode-map}"
  ; Initializing
  (setq comment-multi-line t)
  
   ; Setting up syntax recognition
;  (make-local-variable 'comment-start)
;  (make-local-variable 'comment-end)
;  (make-local-variable 'comment-start-skip)
  (setq comment-start "/* "
    comment-end   " */"
	comment-start-skip "// *")
  
  (setq c-conditional-key php-conditional-key)

  (defvar php-mode-syntax-table php-mode-syntax-table)
  ;; this line makes $ into punctuation instead of a word constituent
  ;; it used to be active, but it killed indenting of case lines that
  ;; begin with '$' (many do).  If anyone has a solution to this
  ;; problem, please let me know.  Of course, you're welcome to
  ;; uncomment this line in your installation.
;  (modify-syntax-entry ?$ "." php-mode-syntax-table)
  
  ;; The above causes Xemacs to handle shell-style comments correctly,
  ;; but fails to work in GNU Emacs which fails to interpret \n as the
  ;; end of the comment.
  (if xemacsp (progn
		(modify-syntax-entry ?# "< b" php-mode-syntax-table)
		(modify-syntax-entry ?\n "> b" php-mode-syntax-table)
		))
  
  (make-local-variable 'font-lock-defaults)
  (setq font-lock-defaults
 	'((php-font-lock-keywords-1
 	   php-font-lock-keywords-2
 	   ;; Comment-out the next line if the font-coloring is too
 	   ;; extreme/ugly for you.
 	   php-font-lock-keywords-3
 	   )
 	  nil				; KEYWORDS-ONLY
 	  T				; CASE-FOLD
 	  nil				; SYNTAX-ALIST
 	  nil				; SYNTAX-BEGIN
 	  (font-lock-syntactic-keywords . php-font-lock-syntactic-keywords)))
  
  (setq font-lock-maximum-decoration t
	case-fold-search t		; PHP vars are case-sensitive
	imenu-generic-expression php-imenu-generic-expression)
  
  ;; Do not force newline at end of file.  Such newlines can cause
  ;; trouble if the PHP file is included in another file before calls
  ;; to header() or cookie().
  (set (make-local-variable 'require-final-newline) nil)
  (set (make-local-variable 'next-line-add-newlines) nil)
  
  ;; PEAR coding standards
  (make-local-hook 'php-mode-pear-hook)
  (add-hook 'php-mode-pear-hook
	    (lambda nil (set (make-local-variable 'tab-width) 4)) nil t)
  (add-hook 'php-mode-pear-hook
	    (lambda nil (set (make-local-variable 'c-basic-offset) 4)) nil t)
  (add-hook 'php-mode-pear-hook
	    (lambda nil (set (make-local-variable 'c-hanging-comment-ender-p) nil)) nil t)
  (add-hook 'php-mode-pear-hook
	    (lambda nil (set (make-local-variable 'indent-tabs-mode) nil)) nil t)
  (add-hook 'php-mode-pear-hook
	    (lambda nil (c-set-offset 'block-open' - )) nil t)
  (add-hook 'php-mode-pear-hook
	    (lambda nil (c-set-offset 'block-close' 0 )) nil t)

  (if (or php-mode-force-pear
	  (and (stringp buffer-file-name)
	       (string-match "PEAR\\|pear"
			     (buffer-file-name))
	       (string-match "\\.php$" (buffer-file-name))))
      (run-hooks 'php-mode-pear-hook))
  
  (run-hooks 'php-mode-user-hook)
  )

;; Make php-mode the default mode for PHP source code buffers.
;;;###autoload
(let ((php-file-patterns-temp php-file-patterns))
  (while php-file-patterns-temp
    (add-to-list 'auto-mode-alist
		  (cons (car php-file-patterns-temp) 'php-mode))
    (setq php-file-patterns-temp (cdr php-file-patterns-temp))))

;; Handle Speedbar
(if php-mode-speedbar-open
    (speedbar 1))
(if (and php-speedbar-config (symbolp 'speedbar))
    (speedbar-add-supported-extension "\\.\\(p?inc\\|php[s34]?\\|phtml\\|p[34]\\)"))

;; Make a menu keymap (with a prompt string)
;; and make it the menu bar item's definition.
(define-key php-mode-map "\C-c\C-d" 'php-search-documentation)
(define-key php-mode-map "\C-c\C-m" 'php-browse-manual)
(define-key php-mode-map "\C-c\C-w" 'php-insert-comment)
(define-key php-mode-map "\C-c\C-e" 'c-comment-edit)
(define-key php-mode-map "\M-\t" 'php-complete-function)
(define-key php-mode-map "\C-c\C-s" 'php-complete-superglobal)
(define-key php-mode-map "\C-c\C-c" 'php-complete-constant)
(define-key php-mode-map "\C-c\C-k" 'php-complete-keyword)
(define-key php-mode-map "\C-c\C-t" 'php-complete-type)


(define-key php-mode-map [menu-bar] (make-sparse-keymap))
(define-key php-mode-map [menu-bar php]
  (cons "PHP" (make-sparse-keymap "PHP")))

;; Define specific subcommands in this menu.
(define-key php-mode-map
  [menu-bar php complete-function]
  '("Complete function name" . php-complete-function))
(define-key php-mode-map
  [menu-bar php complete-keyword]
  '("Complete keyword name" . php-complete-keyword))
(define-key php-mode-map
  [menu-bar php complete-superglobal]
  '("Complete superglobal name" . php-complete-superglobal))
(define-key php-mode-map
  [menu-bar php complete-type]
  '("Complete type name" . php-complete-type))
(define-key php-mode-map
  [menu-bar php complete-constant]
  '("Complete constant name" . php-complete-constant))
(define-key php-mode-map
  [menu-bar php browse-manual]
  '("Browse manual" . php-browse-manual))
(define-key php-mode-map
  [menu-bar php search-documentation]
  '("Search documentation" . php-search-documentation))

;;; PHP constants words
(defvar php-constants
  '("ZEND_THREAD_SAFE" "YPERR_YPSERV" "YPERR_YPERR" "YPERR_YPBIND"
    "YPERR_VERS" "YPERR_RPC" "YPERR_RESRC" "YPERR_PMAP" "YPERR_NOMORE"
    "YPERR_NODOM" "YPERR_MAP" "YPERR_KEY" "YPERR_DOMAIN" "YPERR_BUSY"
    "YPERR_BADDB" "YPERR_BADARGS" "YESEXPR"
    "XSLT_SABOPT_PARSE_PUBLIC_ENTITIES"
    "XSLT_SABOPT_IGNORE_DOC_NOT_FOUND" "XSLT_SABOPT_DISABLE_STRIPPING"
    "XSLT_SABOPT_DISABLE_ADDING_META" "XSLT_OPT_SILENT"
    "XSLT_ERR_UNSUPPORTED_SCHEME" "XSD_UNSIGNEDSHORT"
    "XSD_UNSIGNEDLONG" "XSD_UNSIGNEDINT" "XSD_UNSIGNEDBYTE"
    "XSD_TOKEN" "XSD_TIME" "XSD_STRING" "XSD_SHORT" "XSD_QNAME"
    "XSD_POSITIVEINTEGER" "XSD_NOTATION" "XSD_NORMALIZEDSTRING"
    "XSD_NONPOSITIVEINTEGER" "XSD_NONNEGATIVEINTEGER" "XSD_NMTOKEN"
    "XSD_NEGATIVEINTEGER" "XSD_NCNAME" "XSD_NAME" "XSD_LONG"
    "XSD_LANGUAGE" "XSD_INTEGER" "XSD_INT" "XSD_IDREFS" "XSD_IDREF"
    "XSD_ID" "XSD_HEXBINARY" "XSD_GYEARMONTH" "XSD_GYEAR"
    "XSD_GMONTHDAY" "XSD_GMONTH" "XSD_GDAY" "XSD_FLOAT" "XSD_ENTITYS"
    "XSD_ENTITY" "XSD_DURATION" "XSD_DOUBLE" "XSD_DECIMAL"
    "XSD_DATETIME" "XSD_DATE" "XSD_BYTE" "XSD_BOOLEAN"
    "XSD_BASE64BINARY" "XSD_ANYURI" "XSD_1999_TIMEINSTANT"
    "XPATH_USERS" "XPATH_UNDEFINED" "XPATH_STRING" "XPATH_RANGE"
    "XPATH_POINT" "XPATH_NUMBER" "XPATH_NODESET" "XPATH_LOCATIONSET"
    "XPATH_BOOLEAN" "XML_TEXT_NODE" "XML_PI_NODE"
    "XML_OPTION_TARGET_ENCODING" "XML_OPTION_SKIP_WHITE"
    "XML_OPTION_SKIP_TAGSTART" "XML_OPTION_CASE_FOLDING"
    "XML_NOTATION_NODE" "XML_NAMESPACE_DECL_NODE"
    "XML_LOCAL_NAMESPACE" "XML_HTML_DOCUMENT_NODE"
    "XML_ERROR_UNKNOWN_ENCODING" "XML_ERROR_UNDEFINED_ENTITY"
    "XML_ERROR_UNCLOSED_TOKEN" "XML_ERROR_UNCLOSED_CDATA_SECTION"
    "XML_ERROR_TAG_MISMATCH" "XML_ERROR_SYNTAX"
    "XML_ERROR_RECURSIVE_ENTITY_REF" "XML_ERROR_PARTIAL_CHAR"
    "XML_ERROR_PARAM_ENTITY_REF" "XML_ERROR_NO_MEMORY"
    "XML_ERROR_NO_ELEMENTS" "XML_ERROR_NONE"
    "XML_ERROR_MISPLACED_XML_PI" "XML_ERROR_JUNK_AFTER_DOC_ELEMENT"
    "XML_ERROR_INVALID_TOKEN" "XML_ERROR_INCORRECT_ENCODING"
    "XML_ERROR_EXTERNAL_ENTITY_HANDLING"
    "XML_ERROR_DUPLICATE_ATTRIBUTE" "XML_ERROR_BINARY_ENTITY_REF"
    "XML_ERROR_BAD_CHAR_REF" "XML_ERROR_ATTRIBUTE_EXTERNAL_ENTITY_REF"
    "XML_ERROR_ASYNC_ENTITY" "XML_ENTITY_REF_NODE" "XML_ENTITY_NODE"
    "XML_ENTITY_DECL_NODE" "XML_ELEMENT_NODE" "XML_ELEMENT_DECL_NODE"
    "XML_DTD_NODE" "XML_DOCUMENT_TYPE_NODE" "XML_DOCUMENT_NODE"
    "XML_DOCUMENT_FRAG_NODE" "XML_COMMENT_NODE"
    "XML_CDATA_SECTION_NODE" "XML_ATTRIBUTE_NOTATION"
    "XML_ATTRIBUTE_NODE" "XML_ATTRIBUTE_NMTOKENS"
    "XML_ATTRIBUTE_NMTOKEN" "XML_ATTRIBUTE_IDREFS"
    "XML_ATTRIBUTE_IDREF" "XML_ATTRIBUTE_ID"
    "XML_ATTRIBUTE_ENUMERATION" "XML_ATTRIBUTE_ENTITY"
    "XML_ATTRIBUTE_DECL_NODE" "XML_ATTRIBUTE_CDATA"
    "X509_PURPOSE_SSL_SERVER" "X509_PURPOSE_SSL_CLIENT"
    "X509_PURPOSE_SMIME_SIGN" "X509_PURPOSE_SMIME_ENCRYPT"
    "X509_PURPOSE_NS_SSL_SERVER" "X509_PURPOSE_CRL_SIGN"
    "X509_PURPOSE_ANY" "WUNTRACED" "WNOHANG" "UPLOAD_ERR_PARTIAL"
    "UPLOAD_ERR_OK" "UPLOAD_ERR_NO_FILE" "UPLOAD_ERR_INI_SIZE"
    "UPLOAD_ERR_FORM_SIZE" "UNKNOWN_TYPE" "T_XOR_EQUAL" "T_WHITESPACE"
    "T_WHILE" "T_VARIABLE" "T_VAR" "T_USE" "T_UNSET_CAST" "T_UNSET"
    "T_SWITCH" "T_STRING_VARNAME" "T_STRING_CAST" "T_STRING"
    "T_STATIC" "T_START_HEREDOC" "T_SR_EQUAL" "T_SR" "T_SL_EQUAL"
    "T_SL" "T_RETURN" "T_REQUIRE_ONCE" "T_REQUIRE" "T_PRINT"
    "T_PLUS_EQUAL" "T_PAAMAYIM_NEKUDOTAYIM" "T_OR_EQUAL"
    "T_OPEN_TAG_WITH_ECHO" "T_OPEN_TAG" "T_OLD_FUNCTION"
    "T_OBJECT_OPERATOR" "T_OBJECT_CAST" "T_NUM_STRING" "T_NEW"
    "T_MUL_EQUAL" "T_MOD_EQUAL" "T_ML_COMMENT" "T_MINUS_EQUAL"
    "T_LOGICAL_XOR" "T_LOGICAL_OR" "T_LOGICAL_AND" "T_LNUMBER"
    "T_LIST" "T_LINE" "T_IS_SMALLER_OR_EQUAL" "T_IS_NOT_IDENTICAL"
    "T_IS_NOT_EQUAL" "T_IS_IDENTICAL" "T_IS_GREATER_OR_EQUAL"
    "T_IS_EQUAL" "T_ISSET" "T_INT_CAST" "T_INLINE_HTML"
    "T_INCLUDE_ONCE" "T_INCLUDE" "T_INC" "T_IF" "T_GLOBAL" "T_FUNC_C"
    "T_FUNCTION" "T_FOREACH" "T_FOR" "T_FMT_AMPM" "T_FMT" "T_FILE"
    "T_EXTENDS" "T_EXIT" "T_EVAL" "T_END_HEREDOC" "T_ENDWHILE"
    "T_ENDSWITCH" "T_ENDIF" "T_ENDFOREACH" "T_ENDFOR" "T_ENDDECLARE"
    "T_ENCAPSED_AND_WHITESPACE" "T_EMPTY" "T_ELSEIF" "T_ELSE" "T_ECHO"
    "T_DOUBLE_COLON" "T_DOUBLE_CAST" "T_DOUBLE_ARROW"
    "T_DOLLAR_OPEN_CURLY_BRACES" "T_DO" "T_DNUMBER" "T_DIV_EQUAL"
    "T_DEFAULT" "T_DECLARE" "T_DEC" "T_CURLY_OPEN" "T_CONTINUE"
    "T_CONSTANT_ENCAPSED_STRING" "T_CONST" "T_CONCAT_EQUAL"
    "T_COMMENT" "T_CLOSE_TAG" "T_CLASS_C" "T_CLASS" "T_CHARACTER"
    "T_CASE" "T_BREAK" "T_BOOL_CAST" "T_BOOLEAN_OR" "T_BOOLEAN_AND"
    "T_BAD_CHARACTER" "T_AS" "T_ARRAY_CAST" "T_ARRAY" "T_AND_EQUAL"
    "TYPEVIDEO" "TYPETEXT" "TYPEOTHER" "TYPEMULTIPART" "TYPEMESSAGE"
    "TYPEIMAGE" "TYPEAUDIO" "TYPEAPPLICATION" "TRUE" "true" "THOUSEP"
    "S_IXUSR" "S_IXOTH" "S_IXGRP" "S_IWUSR" "S_IWOTH" "S_IWGRP"
    "S_IRWXU" "S_IRWXO" "S_IRWXG" "S_IRUSR" "S_IROTH" "S_IRGRP"
    "ST_UID" "ST_SILENT" "ST_SET" "STR_PAD_RIGHT" "STR_PAD_LEFT"
    "STR_PAD_BOTH" "STREAM_USE_PATH" "STREAM_REPORT_ERRORS"
    "STREAM_NOTIFY_SEVERITY_WARN" "STREAM_NOTIFY_SEVERITY_INFO"
    "STREAM_NOTIFY_SEVERITY_ERR" "STREAM_NOTIFY_REDIRECTED"
    "STREAM_NOTIFY_PROGRESS" "STREAM_NOTIFY_MIME_TYPE_IS"
    "STREAM_NOTIFY_FILE_SIZE_IS" "STREAM_NOTIFY_FAILURE"
    "STREAM_NOTIFY_CONNECT" "STREAM_NOTIFY_AUTH_RESULT"
    "STREAM_NOTIFY_AUTH_REQUIRED" "STREAM_MUST_SEEK"
    "STREAM_IGNORE_URL" "STREAM_ENFORCE_SAFE_MODE" "STDOUT" "STDIN"
    "STDERR" "SQL_VARCHAR" "SQL_VARBINARY" "SQL_TINYINT"
    "SQL_TIMESTAMP" "SQL_TIME" "SQL_SMALLINT" "SQL_REAL"
    "SQL_ODBC_CURSORS" "SQL_NUMERIC" "SQL_LONGVARCHAR"
    "SQL_LONGVARBINARY" "SQL_KEYSET_SIZE" "SQL_INTEGER" "SQL_FLOAT"
    "SQL_FETCH_NEXT" "SQL_FETCH_FIRST" "SQL_DOUBLE" "SQL_DECIMAL"
    "SQL_DATE" "SQL_CUR_USE_ODBC" "SQL_CUR_USE_IF_NEEDED"
    "SQL_CUR_USE_DRIVER" "SQL_CURSOR_TYPE" "SQL_CURSOR_STATIC"
    "SQL_CURSOR_KEYSET_DRIVEN" "SQL_CURSOR_FORWARD_ONLY"
    "SQL_CURSOR_DYNAMIC" "SQL_CONCUR_VALUES" "SQL_CONCUR_ROWVER"
    "SQL_CONCUR_READ_ONLY" "SQL_CONCUR_LOCK" "SQL_CONCURRENCY"
    "SQL_CHAR" "SQL_BIT" "SQL_BINARY" "SQL_BIGINT" "SO_TYPE"
    "SO_SNDTIMEO" "SO_SNDLOWAT" "SO_SNDBUF" "SO_REUSEADDR"
    "SO_RCVTIMEO" "SO_RCVLOWAT" "SO_RCVBUF" "SO_OOBINLINE"
    "SO_NOSERVER" "SO_LINGER" "SO_KEEPALIVE" "SO_FREE" "SO_ERROR"
    "SO_DONTROUTE" "SO_DEBUG" "SO_BROADCAST" "SORT_STRING"
    "SORT_REGULAR" "SORT_NUMERIC" "SORT_DESC" "SORT_ASC" "SORTTO"
    "SORTSUBJECT" "SORTSIZE" "SORTFROM" "SORTDATE" "SORTCC"
    "SORTARRIVAL" "SOMAXCONN" "SOL_UDP" "SOL_TCP" "SOL_SOCKET"
    "SOCK_STREAM" "SOCK_SEQPACKET" "SOCK_RDM" "SOCK_RAW" "SOCK_DGRAM"
    "SOCKET_EXFULL" "SOCKET_EXDEV" "SOCKET_EWOULDBLOCK"
    "SOCKET_EUSERS" "SOCKET_EUNATCH" "SOCKET_ETOOMANYREFS"
    "SOCKET_ETIMEDOUT" "SOCKET_ETIME" "SOCKET_ESTRPIPE"
    "SOCKET_ESRMNT" "SOCKET_ESPIPE" "SOCKET_ESOCKTNOSUPPORT"
    "SOCKET_ESHUTDOWN" "SOCKET_EROFS" "SOCKET_ERESTART"
    "SOCKET_EREMOTEIO" "SOCKET_EREMOTE" "SOCKET_EREMCHG"
    "SOCKET_EPROTOTYPE" "SOCKET_EPROTONOSUPPORT" "SOCKET_EPROTO"
    "SOCKET_EPIPE" "SOCKET_EPFNOSUPPORT" "SOCKET_EPERM"
    "SOCKET_EOPNOTSUPP" "SOCKET_ENXIO" "SOCKET_ENOTUNIQ"
    "SOCKET_ENOTTY" "SOCKET_ENOTSOCK" "SOCKET_ENOTEMPTY"
    "SOCKET_ENOTDIR" "SOCKET_ENOTCONN" "SOCKET_ENOTBLK"
    "SOCKET_ENOSYS" "SOCKET_ENOSTR" "SOCKET_ENOSR" "SOCKET_ENOSPC"
    "SOCKET_ENOPROTOOPT" "SOCKET_ENONET" "SOCKET_ENOMSG"
    "SOCKET_ENOMEM" "SOCKET_ENOMEDIUM" "SOCKET_ENOLINK"
    "SOCKET_ENOLCK" "SOCKET_ENOENT" "SOCKET_ENODEV" "SOCKET_ENODATA"
    "SOCKET_ENOCSI" "SOCKET_ENOBUFS" "SOCKET_ENOANO" "SOCKET_ENFILE"
    "SOCKET_ENETUNREACH" "SOCKET_ENETRESET" "SOCKET_ENETDOWN"
    "SOCKET_ENAMETOOLONG" "SOCKET_EMULTIHOP" "SOCKET_EMSGSIZE"
    "SOCKET_EMLINK" "SOCKET_EMFILE" "SOCKET_EMEDIUMTYPE"
    "SOCKET_ELOOP" "SOCKET_ELNRNG" "SOCKET_EL3RST" "SOCKET_EL3HLT"
    "SOCKET_EL2NSYNC" "SOCKET_EL2HLT" "SOCKET_EISNAM" "SOCKET_EISDIR"
    "SOCKET_EISCONN" "SOCKET_EIO" "SOCKET_EINVAL" "SOCKET_EINTR"
    "SOCKET_EINPROGRESS" "SOCKET_EIDRM" "SOCKET_EHOSTUNREACH"
    "SOCKET_EHOSTDOWN" "SOCKET_EFAULT" "SOCKET_EEXIST" "SOCKET_EDQUOT"
    "SOCKET_EDESTADDRREQ" "SOCKET_ECONNRESET" "SOCKET_ECONNREFUSED"
    "SOCKET_ECONNABORTED" "SOCKET_ECOMM" "SOCKET_ECHRNG"
    "SOCKET_EBUSY" "SOCKET_EBADSLT" "SOCKET_EBADRQC" "SOCKET_EBADR"
    "SOCKET_EBADMSG" "SOCKET_EBADFD" "SOCKET_EBADF" "SOCKET_EBADE"
    "SOCKET_EALREADY" "SOCKET_EAGAIN" "SOCKET_EAFNOSUPPORT"
    "SOCKET_EADV" "SOCKET_EADDRNOTAVAIL" "SOCKET_EADDRINUSE"
    "SOCKET_EACCES" "SOCKET_E2BIG" "SOAP_RPC"
    "SOAP_PERSISTENCE_SESSION" "SOAP_PERSISTENCE_REQUEST"
    "SOAP_LITERAL" "SOAP_FUNCTIONS_ALL" "SOAP_ENC_OBJECT"
    "SOAP_ENC_ARRAY" "SOAP_ENCODED" "SOAP_DOCUMENT" "SIG_IGN"
    "SIG_ERR" "SIG_DFL" "SIGXFSZ" "SIGXCPU" "SIGWINCH" "SIGVTALRM"
    "SIGUSR2" "SIGUSR1" "SIGURG" "SIGTTOU" "SIGTTIN" "SIGTSTP"
    "SIGTRAP" "SIGTERM" "SIGSYS" "SIGSTOP" "SIGSTKFLT" "SIGSEGV"
    "SIGQUIT" "SIGPWR" "SIGPROF" "SIGPOLL" "SIGPIPE" "SIGKILL"
    "SIGIOT" "SIGIO" "SIGINT" "SIGILL" "SIGHUP" "SIGFPE" "SIGCONT"
    "SIGCLD" "SIGCHLD" "SIGBUS" "SIGBABY" "SIGALRM" "SIGABRT" "SE_UID"
    "SE_NOPREFETCH" "SE_FREE" "SEEK_SET" "SEEK_END" "SEEK_CUR"
    "SA_UNSEEN" "SA_UIDVALIDITY" "SA_UIDNEXT" "SA_RECENT"
    "SA_MESSAGES" "SA_ALL" "RADIXCHAR" "PSPELL_RUN_TOGETHER"
    "PSPELL_NORMAL" "PSPELL_FAST" "PSPELL_BAD_SPELLERS"
    "PREG_SPLIT_OFFSET_CAPTURE" "PREG_SPLIT_NO_EMPTY"
    "PREG_SPLIT_DELIM_CAPTURE" "PREG_SET_ORDER" "PREG_PATTERN_ORDER"
    "PREG_OFFSET_CAPTURE" "PREG_GREP_INVERT" "PM_STR" "PKCS7_TEXT"
    "PKCS7_NOVERIFY" "PKCS7_NOSIGS" "PKCS7_NOINTERN" "PKCS7_NOCHAIN"
    "PKCS7_NOCERTS" "PKCS7_NOATTR" "PKCS7_DETACHED" "PKCS7_BINARY"
    "PHP_VERSION" "PHP_SYSCONFDIR" "PHP_SHLIB_SUFFIX" "PHP_SAPI"
    "PHP_PREFIX" "PHP_OUTPUT_HANDLER_START" "PHP_OUTPUT_HANDLER_END"
    "PHP_OUTPUT_HANDLER_CONT" "PHP_OS" "PHP_NORMAL_READ"
    "PHP_LOCALSTATEDIR" "PHP_LIBDIR" "PHP_EXTENSION_DIR" "PHP_DATADIR"
    "PHP_CONFIG_FILE_SCAN_DIR" "PHP_CONFIG_FILE_PATH" "PHP_BINDIR"
    "PHP_BINARY_READ" "PGSQL_TUPLES_OK" "PGSQL_STATUS_STRING"
    "PGSQL_STATUS_LONG" "PGSQL_SEEK_SET" "PGSQL_SEEK_END"
    "PGSQL_SEEK_CUR" "PGSQL_NUM" "PGSQL_NONFATAL_ERROR"
    "PGSQL_FATAL_ERROR" "PGSQL_EMPTY_QUERY" "PGSQL_DML_STRING"
    "PGSQL_DML_NO_CONV" "PGSQL_DML_EXEC" "PGSQL_DML_ASYNC"
    "PGSQL_COPY_OUT" "PGSQL_COPY_IN" "PGSQL_CONV_IGNORE_NOT_NULL"
    "PGSQL_CONV_IGNORE_DEFAULT" "PGSQL_CONV_FORCE_NULL"
    "PGSQL_CONNECT_FORCE_NEW" "PGSQL_CONNECTION_OK"
    "PGSQL_CONNECTION_BAD" "PGSQL_COMMAND_OK" "PGSQL_BOTH"
    "PGSQL_BAD_RESPONSE" "PGSQL_ASSOC" "PEAR_INSTALL_DIR"
    "PEAR_EXTENSION_DIR" "PATH_SEPARATOR" "PATHINFO_EXTENSION"
    "PATHINFO_DIRNAME" "PATHINFO_BASENAME" "O_WRONLY" "O_TRUNC"
    "O_SYNC" "O_RDWR" "O_RDONLY" "O_NONBLOCK" "O_NOCTTY" "O_NDELAY"
    "O_EXCL" "O_CREAT" "O_ASYNC" "O_APPEND" "OP_SILENT"
    "OP_SHORTCACHE" "OP_SECURE" "OP_READONLY" "OP_PROTOTYPE"
    "OP_HALFOPEN" "OP_EXPUNGE" "OP_DEBUG" "OP_ANONYMOUS"
    "OPENSSL_SSLV23_PADDING" "OPENSSL_PKCS1_PADDING"
    "OPENSSL_PKCS1_OAEP_PADDING" "OPENSSL_NO_PADDING"
    "OPENSSL_KEYTYPE_RSA" "OPENSSL_KEYTYPE_DSA" "OPENSSL_KEYTYPE_DH"
    "ODBC_TYPE" "ODBC_BINMODE_RETURN" "ODBC_BINMODE_PASSTHRU"
    "ODBC_BINMODE_CONVERT" "NULL" "NOEXPR" "NIL"
    "NCURSES_REPORT_MOUSE_POSITION" "NCURSES_KEY_UP"
    "NCURSES_KEY_UNDO" "NCURSES_KEY_SUSPEND" "NCURSES_KEY_SUNDO"
    "NCURSES_KEY_STAB" "NCURSES_KEY_SSUSPEND" "NCURSES_KEY_SSAVE"
    "NCURSES_KEY_SRSUME" "NCURSES_KEY_SRIGHT" "NCURSES_KEY_SRESET"
    "NCURSES_KEY_SREPLACE" "NCURSES_KEY_SREDO" "NCURSES_KEY_SR"
    "NCURSES_KEY_SPRINT" "NCURSES_KEY_SPREVIOUS"
    "NCURSES_KEY_SOPTIONS" "NCURSES_KEY_SNEXT" "NCURSES_KEY_SMOVE"
    "NCURSES_KEY_SMESSAGE" "NCURSES_KEY_SLEFT" "NCURSES_KEY_SIC"
    "NCURSES_KEY_SHOME" "NCURSES_KEY_SHELP" "NCURSES_KEY_SFIND"
    "NCURSES_KEY_SF" "NCURSES_KEY_SEXIT" "NCURSES_KEY_SEOL"
    "NCURSES_KEY_SEND" "NCURSES_KEY_SELECT" "NCURSES_KEY_SDL"
    "NCURSES_KEY_SDC" "NCURSES_KEY_SCREATE" "NCURSES_KEY_SCOPY"
    "NCURSES_KEY_SCOMMAND" "NCURSES_KEY_SCANCEL" "NCURSES_KEY_SBEG"
    "NCURSES_KEY_SAVE" "NCURSES_KEY_RIGHT" "NCURSES_KEY_RESUME"
    "NCURSES_KEY_RESTART" "NCURSES_KEY_RESIZE" "NCURSES_KEY_RESET"
    "NCURSES_KEY_REPLACE" "NCURSES_KEY_REFRESH"
    "NCURSES_KEY_REFERENCE" "NCURSES_KEY_REDO" "NCURSES_KEY_PRINT"
    "NCURSES_KEY_PREVIOUS" "NCURSES_KEY_PPAGE" "NCURSES_KEY_OPTIONS"
    "NCURSES_KEY_OPEN" "NCURSES_KEY_NPAGE" "NCURSES_KEY_NEXT"
    "NCURSES_KEY_MOVE" "NCURSES_KEY_MOUSE" "NCURSES_KEY_MESSAGE"
    "NCURSES_KEY_MARK" "NCURSES_KEY_LL" "NCURSES_KEY_LEFT"
    "NCURSES_KEY_IL" "NCURSES_KEY_IC" "NCURSES_KEY_HELP"
    "NCURSES_KEY_FIND" "NCURSES_KEY_F9" "NCURSES_KEY_F8"
    "NCURSES_KEY_F7" "NCURSES_KEY_F6" "NCURSES_KEY_F5"
    "NCURSES_KEY_F4" "NCURSES_KEY_F3" "NCURSES_KEY_F2"
    "NCURSES_KEY_F12" "NCURSES_KEY_F11" "NCURSES_KEY_F10"
    "NCURSES_KEY_F1" "NCURSES_KEY_F0" "NCURSES_KEY_EXIT"
    "NCURSES_KEY_EOS" "NCURSES_KEY_EOL" "NCURSES_KEY_ENTER"
    "NCURSES_KEY_END" "NCURSES_KEY_EIC" "NCURSES_KEY_DOWN"
    "NCURSES_KEY_DL" "NCURSES_KEY_DC" "NCURSES_KEY_CTAB"
    "NCURSES_KEY_CREATE" "NCURSES_KEY_COPY" "NCURSES_KEY_COMMAND"
    "NCURSES_KEY_CLOSE" "NCURSES_KEY_CLEAR" "NCURSES_KEY_CATAB"
    "NCURSES_KEY_CANCEL" "NCURSES_KEY_C3" "NCURSES_KEY_C1"
    "NCURSES_KEY_BTAB" "NCURSES_KEY_BEG" "NCURSES_KEY_BACKSPACE"
    "NCURSES_KEY_B2" "NCURSES_KEY_A3" "NCURSES_KEY_A1"
    "NCURSES_COLOR_YELLOW" "NCURSES_COLOR_WHITE" "NCURSES_COLOR_RED"
    "NCURSES_COLOR_MAGENTA" "NCURSES_COLOR_GREEN" "NCURSES_COLOR_CYAN"
    "NCURSES_COLOR_BLUE" "NCURSES_COLOR_BLACK" "NCURSES_BUTTON_SHIFT"
    "NCURSES_BUTTON_CTRL" "NCURSES_BUTTON_ALT"
    "NCURSES_BUTTON4_TRIPLE_CLICKED" "NCURSES_BUTTON4_RELEASED"
    "NCURSES_BUTTON4_PRESSED" "NCURSES_BUTTON4_DOUBLE_CLICKED"
    "NCURSES_BUTTON4_CLICKED" "NCURSES_BUTTON3_TRIPLE_CLICKED"
    "NCURSES_BUTTON3_RELEASED" "NCURSES_BUTTON3_PRESSED"
    "NCURSES_BUTTON3_DOUBLE_CLICKED" "NCURSES_BUTTON3_CLICKED"
    "NCURSES_BUTTON2_TRIPLE_CLICKED" "NCURSES_BUTTON2_RELEASED"
    "NCURSES_BUTTON2_PRESSED" "NCURSES_BUTTON2_DOUBLE_CLICKED"
    "NCURSES_BUTTON2_CLICKED" "NCURSES_BUTTON1_TRIPLE_CLICKED"
    "NCURSES_BUTTON1_RELEASED" "NCURSES_BUTTON1_PRESSED"
    "NCURSES_BUTTON1_DOUBLE_CLICKED" "NCURSES_BUTTON1_CLICKED"
    "NCURSES_A_UNDERLINE" "NCURSES_A_STANDOUT" "NCURSES_A_REVERSE"
    "NCURSES_A_PROTECT" "NCURSES_A_NORMAL" "NCURSES_A_INVIS"
    "NCURSES_A_DIM" "NCURSES_A_CHARTEXT" "NCURSES_A_BOLD"
    "NCURSES_A_BLINK" "NCURSES_A_ALTCHARSET"
    "NCURSES_ALL_MOUSE_EVENTS" "M_SQRT2" "M_SQRT1_2" "M_PI_4" "M_PI_2"
    "M_PI" "M_LOG2E" "M_LOG10E" "M_LN2" "M_LN10" "M_E" "M_2_SQRTPI"
    "M_2_PI" "M_1_PI" "MYSQL_NUM" "MYSQL_CLIENT_INTERACTIVE"
    "MYSQL_CLIENT_IGNORE_SPACE" "MYSQL_CLIENT_COMPRESS" "MYSQL_BOTH"
    "MYSQL_ASSOC" "MSG_WAITALL" "MSG_PEEK" "MSG_OOB" "MSG_DONTROUTE"
    "MON_9" "MON_8" "MON_7" "MON_6" "MON_5" "MON_4" "MON_3" "MON_2"
    "MON_12" "MON_11" "MON_10" "MON_1" "MHASH_TIGER160"
    "MHASH_TIGER128" "MHASH_TIGER" "MHASH_SHA256" "MHASH_SHA1"
    "MHASH_RIPEMD160" "MHASH_MD5" "MHASH_MD4" "MHASH_HAVAL256"
    "MHASH_HAVAL224" "MHASH_HAVAL192" "MHASH_HAVAL160"
    "MHASH_HAVAL128" "MHASH_GOST" "MHASH_CRC32B" "MHASH_CRC32"
    "MHASH_ADLER32" "MCRYPT_XTEA" "MCRYPT_WAKE" "MCRYPT_TWOFISH"
    "MCRYPT_TRIPLEDES" "MCRYPT_THREEWAY" "MCRYPT_SKIPJACK"
    "MCRYPT_SERPENT" "MCRYPT_SAFERPLUS" "MCRYPT_SAFER64"
    "MCRYPT_SAFER128" "MCRYPT_RIJNDAEL_256" "MCRYPT_RIJNDAEL_192"
    "MCRYPT_RIJNDAEL_128" "MCRYPT_RC6" "MCRYPT_RC2" "MCRYPT_RAND"
    "MCRYPT_PANAMA" "MCRYPT_MODE_STREAM" "MCRYPT_MODE_OFB"
    "MCRYPT_MODE_NOFB" "MCRYPT_MODE_ECB" "MCRYPT_MODE_CFB"
    "MCRYPT_MODE_CBC" "MCRYPT_MARS" "MCRYPT_LOKI97" "MCRYPT_IDEA"
    "MCRYPT_GOST" "MCRYPT_ENIGNA" "MCRYPT_ENCRYPT"
    "MCRYPT_DEV_URANDOM" "MCRYPT_DEV_RANDOM" "MCRYPT_DES"
    "MCRYPT_DECRYPT" "MCRYPT_CRYPT" "MCRYPT_CAST_256"
    "MCRYPT_CAST_128" "MCRYPT_BLOWFISH_COMPAT" "MCRYPT_BLOWFISH"
    "MCRYPT_ARCFOUR_IV" "MCRYPT_ARCFOUR" "MCRYPT_3DES"
    "MB_OVERLOAD_STRING" "MB_OVERLOAD_REGEX" "MB_OVERLOAD_MAIL"
    "MB_CASE_UPPER" "MB_CASE_TITLE" "MB_CASE_LOWER" "LOG_WARNING"
    "LOG_UUCP" "LOG_USER" "LOG_SYSLOG" "LOG_PID" "LOG_PERROR"
    "LOG_ODELAY" "LOG_NOWAIT" "LOG_NOTICE" "LOG_NEWS" "LOG_NDELAY"
    "LOG_MAIL" "LOG_LPR" "LOG_LOCAL7" "LOG_LOCAL6" "LOG_LOCAL5"
    "LOG_LOCAL4" "LOG_LOCAL3" "LOG_LOCAL2" "LOG_LOCAL1" "LOG_LOCAL0"
    "LOG_KERN" "LOG_INFO" "LOG_ERR" "LOG_EMERG" "LOG_DEBUG"
    "LOG_DAEMON" "LOG_CRON" "LOG_CRIT" "LOG_CONS" "LOG_AUTHPRIV"
    "LOG_AUTH" "LOG_ALERT" "LOCK_UN" "LOCK_SH" "LOCK_NB" "LOCK_EX"
    "LDAP_OPT_TIMELIMIT" "LDAP_OPT_SIZELIMIT"
    "LDAP_OPT_SERVER_CONTROLS" "LDAP_OPT_RESTART" "LDAP_OPT_REFERRALS"
    "LDAP_OPT_PROTOCOL_VERSION" "LDAP_OPT_MATCHED_DN"
    "LDAP_OPT_HOST_NAME" "LDAP_OPT_ERROR_STRING"
    "LDAP_OPT_ERROR_NUMBER" "LDAP_OPT_DEREF" "LDAP_OPT_DEBUG_LEVEL"
    "LDAP_OPT_CLIENT_CONTROLS" "LDAP_DEREF_SEARCHING"
    "LDAP_DEREF_NEVER" "LDAP_DEREF_FINDING" "LDAP_DEREF_ALWAYS"
    "LC_TIME" "LC_NUMERIC" "LC_MONETARY" "LC_MESSAGES" "LC_CTYPE"
    "LC_COLLATE" "LC_ALL" "LATT_UNMARKED" "LATT_NOSELECT"
    "LATT_NOINFERIORS" "LATT_MARKED" "INI_USER" "INI_SYSTEM"
    "INI_PERDIR" "INI_ALL" "INFO_VARIABLES" "INFO_MODULES"
    "INFO_LICENSE" "INFO_GENERAL" "INFO_ENVIRONMENT" "INFO_CREDITS"
    "INFO_CONFIGURATION" "INFO_ALL" "IMG_XPM" "IMG_WBMP" "IMG_PNG"
    "IMG_JPG" "IMG_JPEG" "IMG_GIF" "IMG_EFFECT_REPLACE"
    "IMG_EFFECT_OVERLAY" "IMG_EFFECT_NORMAL" "IMG_EFFECT_ALPHABLEND"
    "IMG_COLOR_TRANSPARENT" "IMG_COLOR_TILED"
    "IMG_COLOR_STYLEDBRUSHED" "IMG_COLOR_STYLED" "IMG_COLOR_BRUSHED"
    "IMG_ARC_ROUNDED" "IMG_ARC_PIE" "IMG_ARC_NOFILL" "IMG_ARC_EDGED"
    "IMG_ARC_CHORD" "IMAGETYPE_TIFF_MM" "IMAGETYPE_TIFF_II"
    "IMAGETYPE_SWF" "IMAGETYPE_SWC" "IMAGETYPE_PSD" "IMAGETYPE_PNG"
    "IMAGETYPE_JPX" "IMAGETYPE_JPEG" "IMAGETYPE_JPC" "IMAGETYPE_JP2"
    "IMAGETYPE_JB2" "IMAGETYPE_IFF" "IMAGETYPE_GIF" "IMAGETYPE_BMP"
    "ICONV_VERSION" "ICONV_IMPL" "HTML_SPECIALCHARS" "HTML_ENTITIES"
    "GMP_ROUND_ZERO" "GMP_ROUND_PLUSINF" "GMP_ROUND_MINUSINF"
    "GLOB_ONLYDIR" "GLOB_NOSORT" "GLOB_NOESCAPE" "GLOB_NOCHECK"
    "GLOB_MARK" "GLOB_BRACE" "GD_BUNDLED" "F_WRLCK" "F_UNLCK"
    "F_SETOWN" "F_SETLKW" "F_SETLK" "F_SETFL" "F_RDLCK" "F_GETOWN"
    "F_GETLK" "F_GETFL" "F_GETFD" "F_DUPFD" "FT_UID" "FT_PREFETCHTEXT"
    "FT_PEEK" "FT_NOT" "FT_INTERNAL" "FTP_TIMEOUT_SEC" "FTP_TEXT"
    "FTP_MOREDATA" "FTP_IMAGE" "FTP_FINISHED" "FTP_FAILED"
    "FTP_BINARY" "FTP_AUTOSEEK" "FTP_AUTORESUME" "FTP_ASCII"
    "FORCE_GZIP" "FORCE_DEFLATE" "FNM_PERIOD" "FNM_PATHNAME"
    "FNM_NOESCAPE" "FALSE" "false" "E_WARNING" "E_USER_WARNING"
    "E_USER_NOTICE" "E_USER_ERROR" "E_PARSE" "E_NOTICE" "E_ERROR"
    "E_CORE_WARNING" "E_CORE_ERROR" "E_COMPILE_WARNING"
    "E_COMPILE_ERROR" "E_ALL" "EXTR_SKIP" "EXTR_REFS"
    "EXTR_PREFIX_SAME" "EXTR_PREFIX_INVALID" "EXTR_PREFIX_IF_EXISTS"
    "EXTR_PREFIX_ALL" "EXTR_OVERWRITE" "EXTR_IF_EXISTS"
    "EXIF_USE_MBSTRING" "ERA_T_FMT" "ERA_D_T_FMT" "ERA_D_FMT" "ERA"
    "ENT_QUOTES" "ENT_NOQUOTES" "ENT_COMPAT" "ENCQUOTEDPRINTABLE"
    "ENCOTHER" "ENCBINARY" "ENCBASE64" "ENC8BIT" "ENC7BIT" "D_T_FMT"
    "D_FMT" "DOMXML_LOAD_VALIDATING" "DOMXML_LOAD_SUBSTITUTE_ENTITIES"
    "DOMXML_LOAD_RECOVERING" "DOMXML_LOAD_PARSING"
    "DOMXML_LOAD_DONT_KEEP_BLANKS" "DOMXML_LOAD_COMPLETE_ATTRS"
    "DIRECTORY_SEPARATOR" "DEFAULT_INCLUDE_PATH" "DAY_7" "DAY_6"
    "DAY_5" "DAY_4" "DAY_3" "DAY_2" "DAY_1" "CURL_NETRC_REQUIRED"
    "CURL_NETRC_OPTIONAL" "CURL_NETRC_IGNORED"
    "CURL_HTTP_VERSION_NONE" "CURL_HTTP_VERSION_1_1"
    "CURL_HTTP_VERSION_1_0" "CURLOPT_WRITEHEADER"
    "CURLOPT_WRITEFUNCTION" "CURLOPT_VERBOSE" "CURLOPT_USERPWD"
    "CURLOPT_USERAGENT" "CURLOPT_URL" "CURLOPT_UPLOAD"
    "CURLOPT_TRANSFERTEXT" "CURLOPT_TIMEVALUE" "CURLOPT_TIMEOUT"
    "CURLOPT_TIMECONDITION" "CURLOPT_STDERR" "CURLOPT_SSL_VERIFYPEER"
    "CURLOPT_SSL_VERIFYHOST" "CURLOPT_SSL_CIPHER_LIST"
    "CURLOPT_SSLVERSION" "CURLOPT_SSLKEYTYPE" "CURLOPT_SSLKEYPASSWD"
    "CURLOPT_SSLKEY" "CURLOPT_SSLENGINE_DEFAULT" "CURLOPT_SSLENGINE"
    "CURLOPT_SSLCERTPASSWD" "CURLOPT_SSLCERT" "CURLOPT_RETURNTRANSFER"
    "CURLOPT_RESUME_FROM" "CURLOPT_REFERER" "CURLOPT_READFUNCTION"
    "CURLOPT_RANGE" "CURLOPT_RANDOM_FILE" "CURLOPT_QUOTE"
    "CURLOPT_PUT" "CURLOPT_PROXYUSERPWD" "CURLOPT_PROXY"
    "CURLOPT_POSTQUOTE" "CURLOPT_POSTFIELDS" "CURLOPT_POST"
    "CURLOPT_PORT" "CURLOPT_PASSWDFUNCTION" "CURLOPT_NOPROGRESS"
    "CURLOPT_NOBODY" "CURLOPT_NETRC" "CURLOPT_MUTE"
    "CURLOPT_MAXREDIRS" "CURLOPT_MAXCONNECTS" "CURLOPT_LOW_SPEED_TIME"
    "CURLOPT_LOW_SPEED_LIMIT" "CURLOPT_KRB4LEVEL" "CURLOPT_INTERFACE"
    "CURLOPT_INFILESIZE" "CURLOPT_INFILE" "CURLOPT_HTTP_VERSION"
    "CURLOPT_HTTPPROXYTUNNEL" "CURLOPT_HTTPHEADER" "CURLOPT_HTTPGET"
    "CURLOPT_HEADERFUNCTION" "CURLOPT_HEADER" "CURLOPT_FTP_USE_EPSV"
    "CURLOPT_FTPPORT" "CURLOPT_FTPLISTONLY" "CURLOPT_FTPASCII"
    "CURLOPT_FTPAPPEND" "CURLOPT_FRESH_CONNECT" "CURLOPT_FORBID_REUSE"
    "CURLOPT_FOLLOWLOCATION" "CURLOPT_FILETIME" "CURLOPT_FILE"
    "CURLOPT_FAILONERROR" "CURLOPT_EGDSOCKET"
    "CURLOPT_DNS_USE_GLOBAL_CACHE" "CURLOPT_DNS_CACHE_TIMEOUT"
    "CURLOPT_CUSTOMREQUEST" "CURLOPT_CRLF" "CURLOPT_COOKIEJAR"
    "CURLOPT_COOKIEFILE" "CURLOPT_COOKIE" "CURLOPT_CONNECTTIMEOUT"
    "CURLOPT_CLOSEPOLICY" "CURLOPT_CAPATH" "CURLOPT_CAINFO"
    "CURLOPT_BINARYTRANSFER" "CURLINFO_TOTAL_TIME"
    "CURLINFO_STARTTRANSFER_TIME" "CURLINFO_SSL_VERIFYRESULT"
    "CURLINFO_SPEED_UPLOAD" "CURLINFO_SPEED_DOWNLOAD"
    "CURLINFO_SIZE_UPLOAD" "CURLINFO_SIZE_DOWNLOAD"
    "CURLINFO_REQUEST_SIZE" "CURLINFO_REDIRECT_TIME"
    "CURLINFO_REDIRECT_COUNT" "CURLINFO_PRETRANSFER_TIME"
    "CURLINFO_NAMELOOKUP_TIME" "CURLINFO_HTTP_CODE"
    "CURLINFO_HEADER_SIZE" "CURLINFO_FILETIME"
    "CURLINFO_EFFECTIVE_URL" "CURLINFO_CONTENT_TYPE"
    "CURLINFO_CONTENT_LENGTH_UPLOAD"
    "CURLINFO_CONTENT_LENGTH_DOWNLOAD" "CURLINFO_CONNECT_TIME"
    "CURLE_WRITE_ERROR" "CURLE_URL_MALFORMAT_USER"
    "CURLE_URL_MALFORMAT" "CURLE_UNSUPPORTED_PROTOCOL"
    "CURLE_UNKNOWN_TELNET_OPTION" "CURLE_TOO_MANY_REDIRECTS"
    "CURLE_TELNET_OPTION_SYNTAX" "CURLE_SSL_PEER_CERTIFICATE"
    "CURLE_SSL_CONNECT_ERROR" "CURLE_READ_ERROR" "CURLE_PARTIAL_FILE"
    "CURLE_OUT_OF_MEMORY" "CURLE_OPERATION_TIMEOUTED" "CURLE_OK"
    "CURLE_OBSOLETE" "CURLE_MALFORMAT_USER" "CURLE_LIBRARY_NOT_FOUND"
    "CURLE_LDAP_SEARCH_FAILED" "CURLE_LDAP_CANNOT_BIND"
    "CURLE_HTTP_RANGE_ERROR" "CURLE_HTTP_POST_ERROR"
    "CURLE_HTTP_PORT_FAILED" "CURLE_HTTP_NOT_FOUND"
    "CURLE_FUNCTION_NOT_FOUND" "CURLE_FTP_WRITE_ERROR"
    "CURLE_FTP_WEIRD_USER_REPLY" "CURLE_FTP_WEIRD_SERVER_REPLY"
    "CURLE_FTP_WEIRD_PASV_REPLY" "CURLE_FTP_WEIRD_PASS_REPLY"
    "CURLE_FTP_WEIRD_227_FORMAT" "CURLE_FTP_USER_PASSWORD_INCORRECT"
    "CURLE_FTP_QUOTE_ERROR" "CURLE_FTP_PORT_FAILED"
    "CURLE_FTP_COULDNT_USE_REST" "CURLE_FTP_COULDNT_STOR_FILE"
    "CURLE_FTP_COULDNT_SET_BINARY" "CURLE_FTP_COULDNT_SET_ASCII"
    "CURLE_FTP_COULDNT_RETR_FILE" "CURLE_FTP_COULDNT_GET_SIZE"
    "CURLE_FTP_CANT_RECONNECT" "CURLE_FTP_CANT_GET_HOST"
    "CURLE_FTP_BAD_DOWNLOAD_RESUME" "CURLE_FTP_ACCESS_DENIED"
    "CURLE_FILE_COULDNT_READ_FILE" "CURLE_FAILED_INIT"
    "CURLE_COULDNT_RESOLVE_PROXY" "CURLE_COULDNT_RESOLVE_HOST"
    "CURLE_COULDNT_CONNECT" "CURLE_BAD_PASSWORD_ENTERED"
    "CURLE_BAD_FUNCTION_ARGUMENT" "CURLE_BAD_CALLING_ORDER"
    "CURLE_ABORTED_BY_CALLBACK" "CURLCLOSEPOLICY_SLOWEST"
    "CURLCLOSEPOLICY_OLDEST" "CURLCLOSEPOLICY_LEAST_TRAFFIC"
    "CURLCLOSEPOLICY_LEAST_RECENTLY_USED" "CURLCLOSEPOLICY_CALLBACK"
    "CRYPT_STD_DES" "CRYPT_SALT_LENGTH" "CRYPT_MD5" "CRYPT_EXT_DES"
    "CRYPT_BLOWFISH" "CRNCYSTR" "CREDITS_SAPI" "CREDITS_QA"
    "CREDITS_MODULES" "CREDITS_GROUP" "CREDITS_GENERAL"
    "CREDITS_FULLPAGE" "CREDITS_DOCS" "CREDITS_ALL" "CP_UID" "CP_MOVE"
    "COUNT_RECURSIVE" "COUNT_NORMAL" "CONNECTION_TIMEOUT"
    "CONNECTION_NORMAL" "CONNECTION_ABORTED" "CODESET" "CL_EXPUNGE"
    "CHAR_MAX" "CASE_UPPER" "CASE_LOWER" "CAL_NUM_CALS"
    "CAL_MONTH_JULIAN_SHORT" "CAL_MONTH_JULIAN_LONG"
    "CAL_MONTH_JEWISH" "CAL_MONTH_GREGORIAN_SHORT"
    "CAL_MONTH_GREGORIAN_LONG" "CAL_MONTH_FRENCH" "CAL_JULIAN"
    "CAL_JEWISH" "CAL_GREGORIAN" "CAL_FRENCH" "CAL_EASTER_ROMAN"
    "CAL_EASTER_DEFAULT" "CAL_EASTER_ALWAYS_JULIAN"
    "CAL_EASTER_ALWAYS_GREGORIAN" "CAL_DOW_SHORT" "CAL_DOW_LONG"
    "CAL_DOW_DAYNO" "ASSERT_WARNING" "ASSERT_QUIET_EVAL"
    "ASSERT_CALLBACK" "ASSERT_BAIL" "ASSERT_ACTIVE" "AM_STR"
    "ALT_DIGITS" "AF_UNIX" "AF_INET" "ABMON_9" "ABMON_8" "ABMON_7"
    "ABMON_6" "ABMON_5" "ABMON_4" "ABMON_3" "ABMON_2" "ABMON_12"
    "ABMON_11" "ABMON_10" "ABMON_1" "ABDAY_7" "ABDAY_6" "ABDAY_5"
    "ABDAY_4" "ABDAY_3" "ABDAY_2" "ABDAY_1"
;;; Pear Constants
;   "PEAR_ZE2" "PEAR_REGISTRY_ERROR_LOCK" "PEAR_REGISTRY_ERROR_FORMAT"
;   "PEAR_REGISTRY_ERROR_FILE" "PEAR_OS" "PEAR_INSTALLER_SKIPPED"
;   "PEAR_INSTALLER_OK" "PEAR_INSTALLER_FAILED" "PEAR_ERROR_TRIGGER"
;   "PEAR_ERROR_RETURN" "PEAR_ERROR_PRINT" "PEAR_ERROR_DIE"
;   "PEAR_ERROR_CALLBACK" "PEAR_DEPENDENCY_UPGRADE_MINOR"
;   "PEAR_DEPENDENCY_UPGRADE_MAJOR" "PEAR_DEPENDENCY_MISSING"
;   "PEAR_DEPENDENCY_CONFLICT" "PEAR_DEPENDENCY_BAD_DEPENDENCY"
;   "PEAR_CONFIG_DEFAULT_VERBOSE" "PEAR_CONFIG_DEFAULT_UMASK"
;   "PEAR_CONFIG_DEFAULT_TEST_DIR" "PEAR_CONFIG_DEFAULT_SIG_TYPE"
;   "PEAR_CONFIG_DEFAULT_SIG_KEYDIR" "PEAR_CONFIG_DEFAULT_SIG_BIN"
;   "PEAR_CONFIG_DEFAULT_PREFERRED_STATE"
;   "PEAR_CONFIG_DEFAULT_PHP_DIR" "PEAR_CONFIG_DEFAULT_PHP_BIN"
;   "PEAR_CONFIG_DEFAULT_MASTER_SERVER"
;   "PEAR_CONFIG_DEFAULT_HTTP_PROXY" "PEAR_CONFIG_DEFAULT_EXT_DIR"
;   "PEAR_CONFIG_DEFAULT_DOC_DIR" "PEAR_CONFIG_DEFAULT_DATA_DIR"
;   "PEAR_CONFIG_DEFAULT_CACHE_TTL" "PEAR_CONFIG_DEFAULT_CACHE_DIR"
;   "PEAR_CONFIG_DEFAULT_BIN_DIR" "PEAR_COMMON_PACKAGE_NAME_PREG"
;   "OS_WINDOWS" "OS_UNIX" "DB_WARNING_READ_ONLY" "DB_WARNING"
;   "DB_TABLEINFO_ORDERTABLE" "DB_TABLEINFO_ORDER" "DB_TABLEINFO_FULL"
;   "DB_PARAM_SCALAR" "DB_PARAM_OPAQUE" "DB_PARAM_MISC" "DB_OK"
;   "DB_GETMODE_ORDERED" "DB_GETMODE_FLIPPED" "DB_GETMODE_ASSOC"
;   "DB_FETCHMODE_ORDERED" "DB_FETCHMODE_OBJECT"
;   "DB_FETCHMODE_FLIPPED" "DB_FETCHMODE_DEFAULT" "DB_FETCHMODE_ASSOC"
;   "DB_ERROR_VALUE_COUNT_ON_ROW" "DB_ERROR_UNSUPPORTED"
;   "DB_ERROR_TRUNCATED" "DB_ERROR_SYNTAX" "DB_ERROR_NOT_LOCKED"
;   "DB_ERROR_NOT_FOUND" "DB_ERROR_NOT_CAPABLE" "DB_ERROR_NOSUCHTABLE"
;   "DB_ERROR_NOSUCHFIELD" "DB_ERROR_NOSUCHDB" "DB_ERROR_NODBSELECTED"
;   "DB_ERROR_NEED_MORE_DATA" "DB_ERROR_MISMATCH"
;   "DB_ERROR_INVALID_NUMBER" "DB_ERROR_INVALID_DSN"
;   "DB_ERROR_INVALID_DATE" "DB_ERROR_INVALID"
;   "DB_ERROR_EXTENSION_NOT_FOUND" "DB_ERROR_DIVZERO"
;   "DB_ERROR_CONSTRAINT" "DB_ERROR_CONNECT_FAILED"
;   "DB_ERROR_CANNOT_DROP" "DB_ERROR_CANNOT_DELETE"
;   "DB_ERROR_CANNOT_CREATE" "DB_ERROR_ALREADY_EXISTS"
;   "DB_ERROR_ACCESS_VIOLATION" "DB_ERROR" "DB_BINMODE_RETURN"
;   "DB_BINMODE_PASSTHRU" "DB_BINMODE_CONVERT" "DB_AUTOQUERY_UPDATE"
;   "DB_AUTOQUERY_INSERT"
  )
  "PHP constants.")

(defvar php-keywords
     ;; "class", "new" and "extends" get special treatment
     ;; "case" and  "default" get special treatment elsewhere
  '("xor" "while" "var" "use" "unset" "switch" "static" "return"
    "require_once" "require" "print" "or" "old_function" "new" "list"
    "isset" "include_once" "include" "if" "global" "function" "foreach"
    "for" "extends" "exit" "eval" "endwhile" "endswitch" "endif"
    "endforeach" "endfor" "enddeclare" "empty" "elseif" "else" "echo" "do"
    "die" "default" "declare" "continue" "const" "class" "cfunction"
    "case" "break" "as" "array" "and" "__LINE__" "__FILE__")
  "PHP keywords.")

(defvar php-functions
  ;; internal functions
  '("zlib_get_coding_type" "zend_version" "zend_logo_guid" "yp_order"
    "yp_next" "yp_match" "yp_master" "yp_get_default_domain" "yp_first"
    "yp_errno" "yp_err_string" "yp_cat" "yp_all" "xslt_setopt"
    "xslt_set_scheme_handlers" "xslt_set_sax_handlers" "xslt_set_object"
    "xslt_set_log" "xslt_set_error_handler" "xslt_set_encoding"
    "xslt_set_base" "xslt_process" "xslt_getopt" "xslt_free" "xslt_error"
    "xslt_errno" "xslt_create" "xslt_backend_version" "xslt_backend_name"
    "xslt_backend_info" "xptr_new_context" "xptr_eval"
    "xpath_register_ns" "xpath_new_context" "xpath_eval_expression"
    "xpath_eval" "xmltree" "xmlrpc_set_type"
    "xmlrpc_server_register_method"
    "xmlrpc_server_register_introspection_callback"
    "xmlrpc_server_destroy" "xmlrpc_server_create"
    "xmlrpc_server_call_method" "xmlrpc_server_add_introspection_data"
    "xmlrpc_parse_method_descriptions" "xmlrpc_is_fault"
    "xmlrpc_get_type" "xmlrpc_encode_request" "xmlrpc_encode"
    "xmlrpc_decode_request" "xmlrpc_decode" "xmldocfile" "xmldoc"
    "xml_set_unparsed_entity_decl_handler"
    "xml_set_start_namespace_decl_handler"
    "xml_set_processing_instruction_handler" "xml_set_object"
    "xml_set_notation_decl_handler" "xml_set_external_entity_ref_handler"
    "xml_set_end_namespace_decl_handler" "xml_set_element_handler"
    "xml_set_default_handler" "xml_set_character_data_handler"
    "xml_parser_set_option" "xml_parser_get_option" "xml_parser_free"
    "xml_parser_create_ns" "xml_parser_create" "xml_parse_into_struct"
    "xml_parse" "xml_get_error_code" "xml_get_current_line_number"
    "xml_get_current_column_number" "xml_get_current_byte_index"
    "xml_error_string" "wordwrap" "wddx_serialize_vars"
    "wddx_serialize_value" "wddx_packet_start" "wddx_packet_end"
    "wddx_deserialize" "wddx_add_vars" "vsprintf" "vprintf" "virtual"
    "version_compare" "var_export" "var_dump" "utf8_encode" "utf8_decode"
    "usort" "usleep" "user_error" "urlencode" "urldecode" "unserialize"
    "unregister_tick_function" "unpack" "unlink" "unixtojd" "uniqid"
    "umask" "uksort" "ucwords" "ucfirst" "uasort" "trim" "trigger_error"
    "touch" "token_name" "token_get_all" "tmpfile" "time" "textdomain"
    "tempnam" "tanh" "tan" "system" "syslog" "symlink"
    "swfbutton_keypress" "substr_replace" "substr_count" "substr"
    "strval" "strtr" "strtoupper" "strtotime" "strtolower" "strtok"
    "strstr" "strspn" "strrpos" "strrev" "strrchr" "strpos" "strncmp"
    "strncasecmp" "strnatcmp" "strnatcasecmp" "strlen" "stristr"
    "stripslashes" "stripcslashes" "strip_tags" "strftime"
    "stream_wrapper_register" "stream_set_write_buffer"
    "stream_set_timeout" "stream_set_blocking" "stream_select"
    "stream_register_wrapper" "stream_get_meta_data"
    "stream_filter_prepend" "stream_filter_append"
    "stream_context_set_params" "stream_context_set_option"
    "stream_context_get_options" "stream_context_create" "strcspn"
    "strcoll" "strcmp" "strchr" "strcasecmp" "str_word_count"
    "str_shuffle" "str_rot13" "str_replace" "str_repeat" "str_pad" "stat"
    "sscanf" "srand" "sqrt" "sql_regcase" "sprintf" "spliti" "split"
    "soundex" "sort" "socket_writev" "socket_write" "socket_strerror"
    "socket_shutdown" "socket_setopt" "socket_set_timeout"
    "socket_set_option" "socket_set_nonblock" "socket_set_blocking"
    "socket_set_block" "socket_sendto" "socket_sendmsg" "socket_send"
    "socket_select" "socket_recvmsg" "socket_recvfrom" "socket_recv"
    "socket_readv" "socket_read" "socket_listen" "socket_last_error"
    "socket_iovec_set" "socket_iovec_free" "socket_iovec_fetch"
    "socket_iovec_delete" "socket_iovec_alloc" "socket_iovec_add"
    "socket_getsockname" "socket_getpeername" "socket_getopt"
    "socket_get_status" "socket_get_option" "socket_create_pair"
    "socket_create_listen" "socket_create" "socket_connect"
    "socket_close" "socket_clear_error" "socket_bind" "socket_accept"
    "snmpwalkoid" "snmpwalk" "snmpset" "snmprealwalk" "snmpget"
    "snmp_set_valueretrieval" "snmp_set_quick_print"
    "snmp_set_oid_numeric_print" "snmp_set_enum_print"
    "snmp_get_valueretrieval" "snmp_get_quick_print" "snmp3_walk"
    "snmp3_set" "snmp3_real_walk" "snmp3_get" "sleep" "sizeof" "sinh"
    "sin" "similar_text" "shuffle" "show_source" "shmop_write"
    "shmop_size" "shmop_read" "shmop_open" "shmop_delete" "shmop_close"
    "shm_remove_var" "shm_remove" "shm_put_var" "shm_get_var"
    "shm_detach" "shm_attach" "shell_exec" "sha1_file" "sha1" "settype"
    "setlocale" "setcookie" "set_time_limit" "set_socket_blocking"
    "set_magic_quotes_runtime" "set_include_path" "set_file_buffer"
    "set_error_handler" "set_content" "session_write_close"
    "session_unset" "session_unregister" "session_start"
    "session_set_save_handler" "session_set_cookie_params"
    "session_save_path" "session_register" "session_regenerate_id"
    "session_name" "session_module_name" "session_is_registered"
    "session_id" "session_get_cookie_params" "session_encode"
    "session_destroy" "session_decode" "session_cache_limiter"
    "session_cache_expire" "serialize" "sem_remove" "sem_release"
    "sem_get" "sem_acquire" "rtrim" "rsort" "round" "rmdir" "rewinddir"
    "rewind" "restore_include_path" "restore_error_handler" "reset"
    "rename" "register_tick_function" "register_shutdown_function"
    "realpath" "readlink" "readgzfile" "readfile" "readdir"
    "read_exif_data" "rawurlencode" "rawurldecode" "range" "rand"
    "rad2deg" "quotemeta" "quoted_printable_decode" "putenv"
    "pspell_suggest" "pspell_store_replacement" "pspell_save_wordlist"
    "pspell_new_personal" "pspell_new_config" "pspell_new"
    "pspell_config_save_repl" "pspell_config_runtogether"
    "pspell_config_repl" "pspell_config_personal" "pspell_config_mode"
    "pspell_config_ignore" "pspell_config_create" "pspell_clear_session"
    "pspell_check" "pspell_add_to_session" "pspell_add_to_personal"
    "proc_open" "proc_close" "printf" "print_r" "prev" "preg_split"
    "preg_replace_callback" "preg_replace" "preg_quote" "preg_match_all"
    "preg_match" "preg_grep" "pow" "posix_uname" "posix_ttyname"
    "posix_times" "posix_strerror" "posix_setuid" "posix_setsid"
    "posix_setpgid" "posix_setgid" "posix_seteuid" "posix_setegid"
    "posix_mkfifo" "posix_kill" "posix_isatty" "posix_getuid"
    "posix_getsid" "posix_getrlimit" "posix_getpwuid" "posix_getpwnam"
    "posix_getppid" "posix_getpid" "posix_getpgrp" "posix_getpgid"
    "posix_getlogin" "posix_getgroups" "posix_getgrnam" "posix_getgrgid"
    "posix_getgid" "posix_geteuid" "posix_getegid" "posix_getcwd"
    "posix_get_last_error" "posix_errno" "posix_ctermid" "pos" "popen"
    "png2wbmp" "pi" "phpversion" "phpinfo" "phpcredits" "php_uname"
    "php_sapi_name" "php_logo_guid" "php_ini_scanned_files" "pg_update"
    "pg_untrace" "pg_unescape_bytea" "pg_tty" "pg_trace"
    "pg_setclientencoding" "pg_set_client_encoding" "pg_send_query"
    "pg_select" "pg_result_status" "pg_result_seek" "pg_result_error"
    "pg_result" "pg_query" "pg_put_line" "pg_port" "pg_ping"
    "pg_pconnect" "pg_options" "pg_numrows" "pg_numfields" "pg_num_rows"
    "pg_num_fields" "pg_meta_data" "pg_lowrite" "pg_lounlink"
    "pg_loreadall" "pg_loread" "pg_loopen" "pg_loimport" "pg_loexport"
    "pg_locreate" "pg_loclose" "pg_lo_write" "pg_lo_unlink" "pg_lo_tell"
    "pg_lo_seek" "pg_lo_read_all" "pg_lo_read" "pg_lo_open"
    "pg_lo_import" "pg_lo_export" "pg_lo_create" "pg_lo_close"
    "pg_last_oid" "pg_last_notice" "pg_last_error" "pg_insert" "pg_host"
    "pg_getlastoid" "pg_get_result" "pg_get_pid" "pg_get_notify"
    "pg_freeresult" "pg_free_result" "pg_fieldtype" "pg_fieldsize"
    "pg_fieldprtlen" "pg_fieldnum" "pg_fieldname" "pg_fieldisnull"
    "pg_field_type" "pg_field_size" "pg_field_prtlen" "pg_field_num"
    "pg_field_name" "pg_field_is_null" "pg_fetch_row" "pg_fetch_result"
    "pg_fetch_object" "pg_fetch_assoc" "pg_fetch_array" "pg_fetch_all"
    "pg_exec" "pg_escape_string" "pg_escape_bytea" "pg_errormessage"
    "pg_end_copy" "pg_delete" "pg_dbname" "pg_copy_to" "pg_copy_from"
    "pg_convert" "pg_connection_status" "pg_connection_reset"
    "pg_connection_busy" "pg_connect" "pg_cmdtuples" "pg_close"
    "pg_clientencoding" "pg_client_encoding" "pg_cancel_query"
    "pg_affected_rows" "pfsockopen" "pclose" "pathinfo" "passthru"
    "parse_url" "parse_str" "parse_ini_file" "pack" "overload"
    "output_reset_rewrite_vars" "output_add_rewrite_var" "ord"
    "openssl_x509_read" "openssl_x509_parse" "openssl_x509_free"
    "openssl_x509_export_to_file" "openssl_x509_export"
    "openssl_x509_checkpurpose" "openssl_x509_check_private_key"
    "openssl_verify" "openssl_sign" "openssl_seal"
    "openssl_public_encrypt" "openssl_public_decrypt"
    "openssl_private_encrypt" "openssl_private_decrypt"
    "openssl_pkey_new" "openssl_pkey_get_public"
    "openssl_pkey_get_private" "openssl_pkey_free"
    "openssl_pkey_export_to_file" "openssl_pkey_export"
    "openssl_pkcs7_verify" "openssl_pkcs7_sign" "openssl_pkcs7_encrypt"
    "openssl_pkcs7_decrypt" "openssl_open" "openssl_get_publickey"
    "openssl_get_privatekey" "openssl_free_key" "openssl_error_string"
    "openssl_csr_sign" "openssl_csr_new" "openssl_csr_export_to_file"
    "openssl_csr_export" "openlog" "opendir" "odbc_tables"
    "odbc_tableprivileges" "odbc_statistics" "odbc_specialcolumns"
    "odbc_setoption" "odbc_rollback" "odbc_result_all" "odbc_result"
    "odbc_procedures" "odbc_procedurecolumns" "odbc_primarykeys"
    "odbc_prepare" "odbc_pconnect" "odbc_num_rows" "odbc_num_fields"
    "odbc_next_result" "odbc_longreadlen" "odbc_gettypeinfo"
    "odbc_free_result" "odbc_foreignkeys" "odbc_field_type"
    "odbc_field_scale" "odbc_field_precision" "odbc_field_num"
    "odbc_field_name" "odbc_field_len" "odbc_fetch_row"
    "odbc_fetch_object" "odbc_fetch_into" "odbc_fetch_array"
    "odbc_execute" "odbc_exec" "odbc_errormsg" "odbc_error" "odbc_do"
    "odbc_data_source" "odbc_cursor" "odbc_connect" "odbc_commit"
    "odbc_columns" "odbc_columnprivileges" "odbc_close_all" "odbc_close"
    "odbc_binmode" "odbc_autocommit" "octdec" "ob_start"
    "ob_list_handlers" "ob_implicit_flush" "ob_iconv_handler"
    "ob_gzhandler" "ob_get_status" "ob_get_level" "ob_get_length"
    "ob_get_flush" "ob_get_contents" "ob_get_clean" "ob_flush"
    "ob_end_flush" "ob_end_clean" "ob_clean" "number_format"
    "nmz_set_sortorder" "nmz_set_sortmethod" "nmz_set_loggingmode"
    "nmz_set_lang" "nmz_set_debugmode" "nmz_search" "nmz_result_score"
    "nmz_result_field" "nmz_result_date" "nmz_open" "nmz_num_hits"
    "nmz_is_lang_ja" "nmz_info" "nmz_get_querytoken" "nmz_get_lang_ctype"
    "nmz_get_lang" "nmz_get_idxnum" "nmz_get_idxname"
    "nmz_get_idx_totalhitnum" "nmz_get_idx_hitnumlist" "nmz_free_result"
    "nmz_fetch_score" "nmz_fetch_field" "nmz_fetch_date"
    "nmz_codeconv_query" "nmz_close" "nl_langinfo" "nl2br" "ngettext"
    "next" "new_xmldoc" "natsort" "natcasesort" "mysql_unbuffered_query"
    "mysql_thread_id" "mysql_tablename" "mysql_table_name" "mysql_stat"
    "mysql_selectdb" "mysql_select_db" "mysql_result"
    "mysql_real_escape_string" "mysql_query" "mysql_ping"
    "mysql_pconnect" "mysql_numrows" "mysql_numfields" "mysql_num_rows"
    "mysql_num_fields" "mysql_listtables" "mysql_listfields"
    "mysql_listdbs" "mysql_list_tables" "mysql_list_processes"
    "mysql_list_fields" "mysql_list_dbs" "mysql_insert_id" "mysql_info"
    "mysql_get_server_info" "mysql_get_proto_info" "mysql_get_host_info"
    "mysql_get_client_info" "mysql_freeresult" "mysql_free_result"
    "mysql_fieldtype" "mysql_fieldtable" "mysql_fieldname"
    "mysql_fieldlen" "mysql_fieldflags" "mysql_field_type"
    "mysql_field_table" "mysql_field_seek" "mysql_field_name"
    "mysql_field_len" "mysql_field_flags" "mysql_fetch_row"
    "mysql_fetch_object" "mysql_fetch_lengths" "mysql_fetch_field"
    "mysql_fetch_assoc" "mysql_fetch_array" "mysql_escape_string"
    "mysql_error" "mysql_errno" "mysql_dbname" "mysql_db_query"
    "mysql_db_name" "mysql_data_seek" "mysql_connect" "mysql_close"
    "mysql_client_encoding" "mysql_affected_rows" "mysql" "mt_srand"
    "mt_rand" "mt_getrandmax" "move_uploaded_file" "money_format"
    "mmcache_unlock" "mmcache_set_session_handlers" "mmcache_rm_page"
    "mmcache_rm" "mmcache_put" "mmcache_lock" "mmcache_load"
    "mmcache_get" "mmcache_gc" "mmcache_encode" "mmcache_cache_result"
    "mmcache_cache_page" "mmcache_cache_output" "mmcache" "mktime"
    "mkdir" "ming_useswfversion" "ming_setscale" "ming_setcubicthreshold"
    "min" "mime_content_type" "microtime" "mhash_keygen_s2k"
    "mhash_get_hash_name" "mhash_get_block_size" "mhash_count" "mhash"
    "method_exists" "metaphone" "memory_get_usage" "mdecrypt_generic"
    "md5_file" "md5" "mcrypt_ofb" "mcrypt_module_self_test"
    "mcrypt_module_open" "mcrypt_module_is_block_mode"
    "mcrypt_module_is_block_algorithm_mode"
    "mcrypt_module_is_block_algorithm"
    "mcrypt_module_get_supported_key_sizes"
    "mcrypt_module_get_algo_key_size" "mcrypt_module_get_algo_block_size"
    "mcrypt_module_close" "mcrypt_list_modes" "mcrypt_list_algorithms"
    "mcrypt_get_key_size" "mcrypt_get_iv_size" "mcrypt_get_cipher_name"
    "mcrypt_get_block_size" "mcrypt_generic_init" "mcrypt_generic_end"
    "mcrypt_generic_deinit" "mcrypt_generic" "mcrypt_encrypt"
    "mcrypt_enc_self_test" "mcrypt_enc_is_block_mode"
    "mcrypt_enc_is_block_algorithm_mode" "mcrypt_enc_is_block_algorithm"
    "mcrypt_enc_get_supported_key_sizes" "mcrypt_enc_get_modes_name"
    "mcrypt_enc_get_key_size" "mcrypt_enc_get_iv_size"
    "mcrypt_enc_get_block_size" "mcrypt_enc_get_algorithms_name"
    "mcrypt_ecb" "mcrypt_decrypt" "mcrypt_create_iv" "mcrypt_cfb"
    "mcrypt_cbc" "mbsubstr" "mbstrrpos" "mbstrpos" "mbstrlen" "mbstrcut"
    "mbsplit" "mbregex_encoding" "mberegi_replace" "mberegi"
    "mbereg_search_setpos" "mbereg_search_regs" "mbereg_search_pos"
    "mbereg_search_init" "mbereg_search_getregs" "mbereg_search_getpos"
    "mbereg_search" "mbereg_replace" "mbereg_match" "mbereg"
    "mb_substr_count" "mb_substr" "mb_substitute_character" "mb_strwidth"
    "mb_strtoupper" "mb_strtolower" "mb_strrpos" "mb_strpos" "mb_strlen"
    "mb_strimwidth" "mb_strcut" "mb_split" "mb_send_mail"
    "mb_regex_set_options" "mb_regex_encoding" "mb_preferred_mime_name"
    "mb_parse_str" "mb_output_handler" "mb_language"
    "mb_internal_encoding" "mb_http_output" "mb_http_input" "mb_get_info"
    "mb_eregi_replace" "mb_eregi" "mb_ereg_search_setpos"
    "mb_ereg_search_regs" "mb_ereg_search_pos" "mb_ereg_search_init"
    "mb_ereg_search_getregs" "mb_ereg_search_getpos" "mb_ereg_search"
    "mb_ereg_replace" "mb_ereg_match" "mb_ereg" "mb_encode_numericentity"
    "mb_encode_mimeheader" "mb_detect_order" "mb_detect_encoding"
    "mb_decode_numericentity" "mb_decode_mimeheader"
    "mb_convert_variables" "mb_convert_kana" "mb_convert_encoding"
    "mb_convert_case" "max" "mail" "magic_quotes_runtime" "ltrim" "lstat"
    "long2ip" "log1p" "log10" "log" "localtime" "localeconv" "linkinfo"
    "link" "levenshtein" "ldap_unbind" "ldap_start_tls" "ldap_sort"
    "ldap_set_rebind_proc" "ldap_set_option" "ldap_search" "ldap_rename"
    "ldap_read" "ldap_parse_result" "ldap_parse_reference"
    "ldap_next_reference" "ldap_next_entry" "ldap_next_attribute"
    "ldap_modify" "ldap_mod_replace" "ldap_mod_del" "ldap_mod_add"
    "ldap_list" "ldap_get_values_len" "ldap_get_values" "ldap_get_option"
    "ldap_get_entries" "ldap_get_dn" "ldap_get_attributes"
    "ldap_free_result" "ldap_first_reference" "ldap_first_entry"
    "ldap_first_attribute" "ldap_explode_dn" "ldap_error" "ldap_errno"
    "ldap_err2str" "ldap_dn2ufn" "ldap_delete" "ldap_count_entries"
    "ldap_connect" "ldap_compare" "ldap_close" "ldap_bind" "ldap_add"
    "lcg_value" "ksort" "krsort" "key_exists" "key" "kakasi_split"
    "kakasi_asorti" "kakasi_asort" "kakasi_arsorti" "kakasi_arsort"
    "kakasi" "juliantojd" "jpeg2wbmp" "join" "jewishtojd" "jdtounix"
    "jdtojulian" "jdtojewish" "jdtogregorian" "jdtofrench" "jdmonthname"
    "jddayofweek" "is_writeable" "is_writable" "is_uploaded_file"
    "is_subclass_of" "is_string" "is_scalar" "is_resource" "is_real"
    "is_readable" "is_object" "is_numeric" "is_null" "is_nan" "is_long"
    "is_link" "is_integer" "is_int" "is_infinite" "is_float" "is_finite"
    "is_file" "is_executable" "is_double" "is_dir" "is_callable"
    "is_bool" "is_array" "is_a" "iptcparse" "iptcembed" "ip2long"
    "intval" "ini_set" "ini_restore" "ini_get_all" "ini_get" "ini_alter"
    "in_array" "import_request_variables" "implode" "imap_utf8"
    "imap_utf7_encode" "imap_utf7_decode" "imap_unsubscribe"
    "imap_undelete" "imap_uid" "imap_timeout" "imap_thread"
    "imap_subscribe" "imap_status" "imap_sort" "imap_setflag_full"
    "imap_setacl" "imap_set_quota" "imap_search" "imap_scanmailbox"
    "imap_scan" "imap_rfc822_write_address" "imap_rfc822_parse_headers"
    "imap_rfc822_parse_adrlist" "imap_reopen" "imap_renamemailbox"
    "imap_rename" "imap_qprint" "imap_ping" "imap_open" "imap_num_recent"
    "imap_num_msg" "imap_msgno" "imap_mime_header_decode"
    "imap_mailboxmsginfo" "imap_mail_move" "imap_mail_copy"
    "imap_mail_compose" "imap_mail" "imap_lsub" "imap_listsubscribed"
    "imap_listmailbox" "imap_list" "imap_last_error" "imap_headers"
    "imap_headerinfo" "imap_header" "imap_getsubscribed"
    "imap_getmailboxes" "imap_get_quotaroot" "imap_get_quota"
    "imap_fetchtext" "imap_fetchstructure" "imap_fetchheader"
    "imap_fetchbody" "imap_fetch_overview" "imap_expunge" "imap_errors"
    "imap_deletemailbox" "imap_delete" "imap_createmailbox" "imap_create"
    "imap_close" "imap_clearflag_full" "imap_check" "imap_bodystruct"
    "imap_body" "imap_binary" "imap_base64" "imap_append" "imap_alerts"
    "imap_8bit" "imagewbmp" "imagetypes" "imagettftext" "imagettfbbox"
    "imagetruecolortopalette" "imagesy" "imagesx" "imagestringup"
    "imagestring" "imagesettile" "imagesetthickness" "imagesetstyle"
    "imagesetpixel" "imagesetbrush" "imagesavealpha" "imagerotate"
    "imagerectangle" "imagepstext" "imagepsslantfont" "imagepsloadfont"
    "imagepsfreefont" "imagepsextendfont" "imagepsencodefont"
    "imagepsbbox" "imagepolygon" "imagepng" "imagepalettecopy"
    "imageloadfont" "imageline" "imagelayereffect" "imagejpeg"
    "imageistruecolor" "imageinterlace" "imagegd2" "imagegd"
    "imagegammacorrect" "imagefttext" "imageftbbox" "imagefontwidth"
    "imagefontheight" "imagefilltoborder" "imagefilledrectangle"
    "imagefilledpolygon" "imagefilledellipse" "imagefilledarc"
    "imagefill" "imageellipse" "imagedestroy" "imagedashedline"
    "imagecreatetruecolor" "imagecreatefromxpm" "imagecreatefromxbm"
    "imagecreatefromwbmp" "imagecreatefromstring" "imagecreatefrompng"
    "imagecreatefromjpeg" "imagecreatefromgif" "imagecreatefromgd2part"
    "imagecreatefromgd2" "imagecreatefromgd" "imagecreate"
    "imagecopyresized" "imagecopyresampled" "imagecopymergegray"
    "imagecopymerge" "imagecopy" "imagecolortransparent"
    "imagecolorstotal" "imagecolorsforindex" "imagecolorset"
    "imagecolorresolvealpha" "imagecolorresolve" "imagecolormatch"
    "imagecolorexactalpha" "imagecolorexact" "imagecolordeallocate"
    "imagecolorclosesthwb" "imagecolorclosestalpha" "imagecolorclosest"
    "imagecolorat" "imagecolorallocatealpha" "imagecolorallocate"
    "imagecharup" "imagechar" "imagearc" "imageantialias"
    "imagealphablending" "image_type_to_mime_type" "image2wbmp"
    "ignore_user_abort" "iconv_set_encoding" "iconv_get_encoding" "iconv"
    "i18n_mime_header_encode" "i18n_mime_header_decode"
    "i18n_ja_jp_hantozen" "i18n_internal_encoding" "i18n_http_output"
    "i18n_http_input" "i18n_discover_encoding" "i18n_convert" "hypot"
    "htmlspecialchars" "htmlentities" "html_entity_decode"
    "html_doc_file" "html_doc" "highlight_string" "highlight_file"
    "hexdec" "hebrevc" "hebrev" "headers_sent" "header" "gzwrite"
    "gzuncompress" "gztell" "gzseek" "gzrewind" "gzread" "gzputs"
    "gzpassthru" "gzopen" "gzinflate" "gzgetss" "gzgets" "gzgetc"
    "gzfile" "gzeof" "gzencode" "gzdeflate" "gzcompress" "gzclose"
    "gregoriantojd" "gmstrftime" "gmp_xor" "gmp_sub" "gmp_strval"
    "gmp_sqrtrem" "gmp_sqrt" "gmp_sign" "gmp_setbit" "gmp_scan1"
    "gmp_scan0" "gmp_random" "gmp_prob_prime" "gmp_powm" "gmp_pow"
    "gmp_popcount" "gmp_perfect_square" "gmp_or" "gmp_neg" "gmp_mul"
    "gmp_mod" "gmp_legendre" "gmp_jacobi" "gmp_invert" "gmp_intval"
    "gmp_init" "gmp_hamdist" "gmp_gcdext" "gmp_gcd" "gmp_fact"
    "gmp_divexact" "gmp_div_r" "gmp_div_qr" "gmp_div_q" "gmp_div"
    "gmp_com" "gmp_cmp" "gmp_clrbit" "gmp_and" "gmp_add" "gmp_abs"
    "gmmktime" "gmdate" "glob" "gettype" "gettimeofday" "gettext"
    "getservbyport" "getservbyname" "getrusage" "getrandmax"
    "getprotobynumber" "getprotobyname" "getopt" "getmyuid" "getmypid"
    "getmyinode" "getmygid" "getmxrr" "getlastmod" "getimagesize"
    "gethostbynamel" "gethostbyname" "gethostbyaddr" "getenv" "getdate"
    "getcwd" "getallheaders" "get_resource_type" "get_required_files"
    "get_parent_class" "get_object_vars" "get_meta_tags"
    "get_magic_quotes_runtime" "get_magic_quotes_gpc"
    "get_loaded_extensions" "get_included_files" "get_include_path"
    "get_html_translation_table" "get_extension_funcs" "get_defined_vars"
    "get_defined_functions" "get_defined_constants"
    "get_declared_classes" "get_current_user" "get_class_vars"
    "get_class_methods" "get_class" "get_cfg_var" "get_browser" "gd_info"
    "fwrite" "function_exists" "func_num_args" "func_get_args"
    "func_get_arg" "ftruncate" "ftp_systype" "ftp_ssl_connect" "ftp_size"
    "ftp_site" "ftp_set_option" "ftp_rmdir" "ftp_rename" "ftp_rawlist"
    "ftp_quit" "ftp_pwd" "ftp_put" "ftp_pasv" "ftp_nlist" "ftp_nb_put"
    "ftp_nb_get" "ftp_nb_fput" "ftp_nb_fget" "ftp_nb_continue"
    "ftp_mkdir" "ftp_mdtm" "ftp_login" "ftp_get_option" "ftp_get"
    "ftp_fput" "ftp_fget" "ftp_exec" "ftp_delete" "ftp_connect"
    "ftp_close" "ftp_chdir" "ftp_cdup" "ftok" "ftell" "fstat" "fsockopen"
    "fseek" "fscanf" "frenchtojd" "fread" "fputs" "fpassthru" "fopen"
    "fnmatch" "fmod" "flush" "floor" "flock" "floatval" "filetype"
    "filesize" "fileperms" "fileowner" "filemtime" "fileinode"
    "filegroup" "filectime" "fileatime" "file_get_contents" "file_exists"
    "file" "fgetss" "fgets" "fgetcsv" "fgetc" "fflush" "feof" "fclose"
    "ezmlm_hash" "extract" "extension_loaded" "expm1" "explode" "exp"
    "exif_thumbnail" "exif_tagname" "exif_read_data" "exif_imagetype"
    "exec" "escapeshellcmd" "escapeshellarg" "error_reporting"
    "error_log" "eregi_replace" "eregi" "ereg_replace" "ereg" "end"
    "easter_days" "easter_date" "each" "doubleval" "domxml_xslt_version"
    "domxml_xslt_stylesheet_file" "domxml_xslt_stylesheet_doc"
    "domxml_xslt_stylesheet" "domxml_xslt_result_dump_mem"
    "domxml_xslt_result_dump_file" "domxml_xslt_process" "domxml_xmltree"
    "domxml_version" "domxml_unlink_node"
    "domxml_substitute_entities_default" "domxml_setattr"
    "domxml_set_attribute" "domxml_root" "domxml_parser_start_element"
    "domxml_parser_start_document" "domxml_parser_processing_instruction"
    "domxml_parser_namespace_decl" "domxml_parser_get_document"
    "domxml_parser_entity_reference" "domxml_parser_end_element"
    "domxml_parser_end_document" "domxml_parser_end"
    "domxml_parser_comment" "domxml_parser_characters"
    "domxml_parser_cdata_section" "domxml_parser_add_chunk"
    "domxml_parser" "domxml_open_mem" "domxml_open_file"
    "domxml_node_unlink_node" "domxml_node_set_namespace"
    "domxml_node_set_content" "domxml_node_new_child"
    "domxml_node_has_attributes" "domxml_node_get_content"
    "domxml_node_children" "domxml_node_attributes"
    "domxml_node_add_namespace" "domxml_node" "domxml_new_xmldoc"
    "domxml_new_doc" "domxml_new_child" "domxml_html_dump_mem"
    "domxml_getattr" "domxml_get_attribute" "domxml_elem_set_attribute"
    "domxml_elem_get_attribute" "domxml_dumpmem" "domxml_dump_node"
    "domxml_dump_mem_file" "domxml_dump_mem" "domxml_doc_xinclude"
    "domxml_doc_validate" "domxml_doc_set_root" "domxml_doc_get_root"
    "domxml_doc_get_elements_by_tagname" "domxml_doc_get_element_by_id"
    "domxml_doc_document_element" "domxml_doc_add_root" "domxml_children"
    "domxml_attributes" "domxml_add_root" "dngettext" "dl"
    "diskfreespace" "disk_total_space" "disk_free_space" "dirname" "dir"
    "dio_write" "dio_truncate" "dio_tcsetattr" "dio_stat" "dio_seek"
    "dio_read" "dio_open" "dio_fcntl" "dio_close" "dgettext" "deg2rad"
    "defined" "define_syslog_variables" "define" "decoct" "dechex"
    "decbin" "debug_zval_dump" "debug_backtrace" "deaggregate"
    "dcngettext" "dcgettext" "dbx_sort" "dbx_query" "dbx_escape_string"
    "dbx_error" "dbx_connect" "dbx_compare" "dbx_close" "dba_sync"
    "dba_replace" "dba_popen" "dba_optimize" "dba_open" "dba_nextkey"
    "dba_list" "dba_insert" "dba_handlers" "dba_firstkey" "dba_fetch"
    "dba_exists" "dba_delete" "dba_close" "date" "current" "curl_version"
    "curl_setopt" "curl_init" "curl_getinfo" "curl_exec" "curl_error"
    "curl_errno" "curl_close" "ctype_xdigit" "ctype_upper" "ctype_space"
    "ctype_punct" "ctype_print" "ctype_lower" "ctype_graph" "ctype_digit"
    "ctype_cntrl" "ctype_alpha" "ctype_alnum" "crypt" "create_function"
    "crc32" "crack_opendict" "crack_getlastmessage" "crack_closedict"
    "crack_check" "count_chars" "count" "cosh" "cos" "copy"
    "convert_cyr_string" "constant" "connection_status"
    "connection_aborted" "compact" "closelog" "closedir" "clearstatcache"
    "class_exists" "chunk_split" "chr" "chown" "chop" "chmod" "chgrp"
    "checkdnsrr" "checkdate" "chdir" "chasen_split" "chasen" "ceil"
    "call_user_method_array" "call_user_method" "call_user_func_array"
    "call_user_func" "cal_to_jd" "cal_info" "cal_from_jd"
    "cal_days_in_month" "bzwrite" "bzread" "bzopen" "bzflush" "bzerrstr"
    "bzerror" "bzerrno" "bzdecompress" "bzcompress" "bzclose"
    "bindtextdomain" "bindec" "bind_textdomain_codeset" "bin2hex" "bcsub"
    "bcsqrt" "bcscale" "bcpow" "bcmul" "bcmod" "bcdiv" "bccomp" "bcadd"
    "basename" "base_convert" "base64_encode" "base64_decode" "atanh"
    "atan2" "atan" "assert_options" "assert" "asort" "asinh" "asin"
    "arsort" "array_walk" "array_values" "array_unshift" "array_unique"
    "array_sum" "array_splice" "array_slice" "array_shift" "array_search"
    "array_reverse" "array_reduce" "array_rand" "array_push" "array_pop"
    "array_pad" "array_multisort" "array_merge_recursive" "array_merge"
    "array_map" "array_keys" "array_key_exists" "array_intersect_assoc"
    "array_intersect" "array_flip" "array_filter" "array_fill"
    "array_diff_assoc" "array_diff" "array_count_values" "array_chunk"
    "array_change_key_case" "apache_setenv" "apache_response_headers"
    "apache_request_headers" "apache_note" "apache_lookup_uri"
    "apache_getenv" "apache_get_version" "apache_get_modules"
    "aggregation_info" "aggregate_properties_by_regexp"
    "aggregate_properties_by_list" "aggregate_properties"
    "aggregate_methods_by_regexp" "aggregate_methods_by_list"
    "aggregate_methods" "aggregate" "addslashes" "addcslashes" "acosh"
    "acos" "abs" "_mmcache_output_handler" "_mmcache_loader_line"
    "_mmcache_loader_file" "_"
    ;; Pear functions
    "xml_rpc_server_methodsignature" "xml_rpc_server_methodhelp"
    "xml_rpc_server_listmethods" "xml_rpc_server_debugmsg" "xml_rpc_se"
    "xml_rpc_lookup_entity" "xml_rpc_iso8601_encode" "xml_rpc_iso8601_decode"
    "xml_rpc_entity_decode" "xml_rpc_encode" "xml_rpc_ee" "xml_rpc_dh"
    "xml_rpc_decode" "xml_rpc_cd" "_pear_call_destructors" )
  "PHP functions.")

(defvar php-identifier
  (eval-when-compile
    '"[a-zA-Z\_\x7f-\xff][a-zA-Z0-9\_\x7f-\xff]*")
  "Characters in a PHP identifier.")

(defvar php-types
  '("array" "boolean" "bool" "char" "const" "double" "float"
    "int" "integer" "long" "mixed" "object" "real" "string")
  "PHP types.")

(defvar php-superglobals
  '("_GET" "_POST" "_COOKIE" "_SESSION" "_ENV" "_SERVER" "_FILES" "_REQUEST" "GLOBALS")
  "PHP superglobal variables.")

(defvar php-functions-alist
  (mapcar
   (lambda(word)
     (cons word nil))
   php-functions)
  "An association list of the PHP functions for completion use.")

(defvar php-keywords-alist
  (mapcar
   (lambda(word)
     (cons word nil))
   php-keywords)
  "An association list of the PHP keywords for completion use.")

(defvar php-constants-alist
  (mapcar
   (lambda(word)
     (cons word nil))
   php-constants)
  "An association list of the PHP constants for completion use.")

(defvar php-types-alist
  (mapcar
   (lambda(word)
     (cons word nil))
   php-types)
  "An association list of the PHP types for completion use.")

(defvar php-superglobals-alist
          (mapcar
           (lambda(word)
             (cons (concat word "[]")  nil))
           php-superglobals)
  "An association list of the PHP superglobals for completion use.")

;;; internal
(defun php-list-to-regexp(start wordlist end)
  "Takes a list and returns the regexp \\(elem1\\|elem2\\|...\\)"
  (let ((regexp start))
    (mapcar (lambda(elem)
          (setq regexp (concat regexp elem "\\|")))
        wordlist)
    (concat (substring regexp 0 -2) ; cutting the last "\\|"
        end)
    ))

;; Set up font locking
(defvar php-font-lock-keywords-1
  (list
   ;; Fontify constants
   (cons (php-list-to-regexp "\\<\\(" php-constants "\\)\\>") font-lock-constant-face)
   ;; Fontify keywords
   (cons (php-list-to-regexp "\\<\\(" php-keywords "\\)\\>") font-lock-keyword-face)
   ;; Fontify internal functions
   (cons (php-list-to-regexp  "\\<\\(" php-functions "\\)\\>") font-lock-function-name-face)
   ;; Fontify keywords and targets, and case default tags.
   (list "\\<\\(break\\|case\\|continue\\)\\>[ \t]*\\(-?\\(?:\\sw\\|\\s_\\)+\\)?"
	 '(1 font-lock-keyword-face) '(2 font-lock-constant-face t t))
   ;; This must come after the one for keywords and targets.
   '(":" ("^[ \t]*\\(\\(?:\\sw\\|\\s_\\)+\\)[ \t]*:[ \t]*$"
	  (beginning-of-line) (end-of-line)
	  (1 font-lock-constant-face)))
   
   ;; treat 'print' as keyword only when not used like a function name
   '("\\<print\\s-*(" . default)
   '("\\<print\\>" . font-lock-keyword-face)
   
   ;; Fontify PHP tag
   '("<\\?\\(php\\)?" . font-lock-constant-face)
   '("\\?>" . font-lock-constant-face)
   
   ;; Fontify ASP-style tag
   '("<\\%\\(=\\)?" . font-lock-constant-face)
   '("\\%>" . font-lock-constant-face)

   )
  "Subdued level highlighting for PHP mode.")

(defvar php-font-lock-keywords-2
  (append
   php-font-lock-keywords-1
   (list
    
    ;; Fontify class declaration
    '("^[ \t]*\\(class\\)[ \t]*\\(\\(?:\\sw\\|\\s_\\)+\\)?"
      (1 font-lock-keyword-face) (2 font-lock-type-face nil t))
    
    ;; Fontify function declaration
    '("^[ \t]*\\(function\\)\\s-+&?\\(\\(?:\\sw\\|\\s_\\)+\\)\\s-*("
      (1 font-lock-keyword-face)
      (2 font-lock-function-name-face nil t))
    
    ;; handle several words specially, to include following word,
    ;; thereby excluding it from unknown-symbol checks later
    '("\\<\\(new\\|extends\\)\\s-+\\$?\\(\\(?:\\sw\\|\\s_\\)+\\)"
      (1 font-lock-keyword-face) (2 font-lock-type-face))
    ))
  "Medium level highlighting for PHP mode.")

(defvar php-font-lock-keywords-3
  (append
   php-font-lock-keywords-2
   (list
    
    ;; <word> or </word> for HTML
    '("</?\\sw+[^>]*>" . font-lock-constant-face)

    ;; HTML entities
    '("&\\w+;" . font-lock-variable-name-face)

    ;; warn about '$' immediately after ->
    '("\\$\\(?:\\sw\\|\\s_\\)+->\\s-*\\(\\$\\)\\(\\(?:\\sw\\|\\s_\\)+\\)"
      (1 font-lock-warning-face) (2 default))
    
    ;; warn about $word.word -- it could be a valid concatenation,
    ;; but without any spaces we'll assume $word->word was meant.
    '("\\$\\(?:\\sw\\|\\s_\\)+\\(\\.\\)\\sw"
      1 font-lock-warning-face)
    
    ;; Warn about ==> instead of =>
    '("==+>" . font-lock-warning-face)
    
    ;; exclude casts from bare-word treatment
    `(,(php-list-to-regexp "(\\(" php-types "\\))")
      1 font-lock-type-face)
    
    ;; Fontify variables and function calls
    '("\\$\\(this\\)\\W" (1 font-lock-constant-face nil nil)) ; "this" as constant
    `(,(php-list-to-regexp "\\$\\(" php-superglobals "\\)\\W")
      (1 font-lock-constant-face nil nil)) ; $_GET & co
    '("\\$\\(\\(?:\\sw\\|\\s_\\)+\\)" (1 font-lock-variable-name-face)) ; $variable
    '("->\\(\\(?:\\sw\\|\\s_\\)+\\)" (1 font-lock-variable-name-face t t)) ; ->variable
    '("->\\(\\(?:\\sw\\|\\s_\\)+\\)\\s-*(" . (1 default t t)) ; ->function_call
    '("\\(?:\\sw\\|\\s_\\)+::\\(?:\\sw\\|\\s_\\)+\\s-*(" . default) ; class::method call
    '("\\<\\(?:\\sw\\|\\s_\\)+\\s-*[[(]" . default)	; word( or word[
    '("\\<[0-9]+" . default)		; number (also matches word)

    ;; Warn on any words not already fontified
    '("\\<\\(?:\\sw\\|\\s_\\)+\\>" . font-lock-warning-face)
    ))
  "Gauchy level highlighting for PHP mode.")

(defvar php-font-lock-syntactic-keywords
  (if xemacsp nil
    ;; Mark shell-style comments.  font-lock handles this in a
    ;; separate pass from normal syntactic scanning (somehow), so we
    ;; get a chance to mark these in addition to C and C++ style
    ;; comments.  This only works in GNU Emacs, not Xemacs 21 which
    ;; seems to ignore this same code if we try to use it.
    (list
     ;; Mark _all_ # chars as being comment-start.  That will be
     ;; ignored when inside a quoted string.
     '("\\(\#\\)"
       (1 (11 . nil)))
     ;; Mark all newlines ending a line with # as being comment-end.
     ;; This causes a problem, premature end-of-comment, when '#'
     ;; appears inside a multiline C-style comment.  Oh well.
     '("#.*\\([\n]\\)"
       (1 (12 . nil)))
     )))

;; Define the imenu-generic-expression for PHP mode.
;; To use, execute M-x imenu, then click on Functions or Classes,
;; then select given function/class name to go to its definition.
;; [Contributed by Gerrit Riessen]
(defvar php-imenu-generic-expression
 '(
   ("Functions"
    "\\(^\\|\\s-\\)function\\s-+\\(\\sw+\\)\\s-*(" 2)
   ("Classes"
    "\\(^\\|\\s-\\)class\\s-+\\(\\sw+\\)\\s-*" 2)
   )
 "Imenu generic expression for PHP Mode. See `imenu-generic-expression'."
 )

;; Add "foreach" to conditional introducing keywords
(defconst php-conditional-key nil)
(let ((all-kws "for\\|if\\|do\\|else\\|while\\|switch")
      (front   "\\<\\(")
      (back    "\\)\\>[^_]"))
  (setq php-conditional-key (concat front all-kws "\\|foreach" back)))

;;; Function name completion (this functions import from css-mode.el)
(defun php-complete-at-point()
  "If point is at the end of a property name: returns it."
  (let ((end (point))
    (start (+ (re-search-backward "[^_-A-Za-z]") 1)))
    (goto-char end)
    (buffer-substring start end)))

; internal
(defun php-maximum-common(alt1 alt2)
  "Returns the maximum common starting substring of alt1 and alt2."
  (let* ((maxlen (min (length alt1) (length alt2)))
     (alt1 (substring alt1 0 maxlen))
     (alt2 (substring alt2 0 maxlen)))
    (while (not (string= (substring alt1 0 maxlen)
             (substring alt2 0 maxlen)))
      (setq maxlen (- maxlen 1)))
    (substring alt1 0 maxlen)))

; internal
(defun php-common-beginning(alts)
  "Returns the maximum common starting substring of all alts elements."
  (let ((common (car alts)))
    (dolist (alt (cdr alts) common)
      (setq common (php-maximum-common alt common)))))

(defun php-complete-frame(completions)
  ; This code stolen from message.el. Kudos to larsi.
  (let ((cur (current-buffer)))
    (pop-to-buffer "*Completions*")
    (buffer-disable-undo (current-buffer))
    (let ((buffer-read-only nil))
      (erase-buffer)
      (let ((standard-output (current-buffer)))
    (display-completion-list (sort completions 'string<)))
      (goto-char (point-min))
      (pop-to-buffer cur))))

(defun php-complete(alist)
  "Complete the function name, keywords, constants, types
at the point from known PHP words."
  (interactive)
  (let* ((word   (php-complete-at-point))
     (alts   (all-completions word alist))
     (wordlen (length word)))
    (if (= (length alts) 1)
    (insert (substring (car alts) wordlen))
      (let ((beg (php-common-beginning alts)))
    (if (not (string= beg word))
        (insert (substring beg wordlen))
      (insert (substring
           (completing-read "Completion: " alist nil
                    nil word)
           wordlen)))))))

(defun php-complete-constant ()
  "Complete the constants name at the point from known PHP constants."
  (interactive)
  (php-complete php-constants-alist))

(defun php-complete-function ()
  "Complete the function name at the point from known PHP functions."
  (interactive)
  (php-complete php-functions-alist))

(defun php-complete-keyword ()
  "Complete the keyword at the point from known PHP keywords."
  (interactive)
  (php-complete php-keywords-alist))

(defun php-complete-superglobal ()
  "Complete the superglobal variable name at the point from known PHP superglobals."
  (interactive)
  (php-complete php-superglobals-alist))

(defun php-complete-type ()
  "Complete the types name at the point from known PHP types."
  (interactive)
  (php-complete php-types-alist))


;; Define function documentation function
(defun php-search-documentation ()
  "Search PHP documentation for the word at the point."
  (interactive)
  (browse-url (concat php-search-url (current-word t)))
  )

;; Define function for browsing manual
(defun php-browse-manual ()
  "Bring up manual for PHP."
  (interactive)
  (browse-url php-manual-url)
  )


;; Create "default" symbol for GNU Emacs so that both Xemacs and GNU
;; emacs can refer to the default face by a variable named "default".
(unless (boundp 'default)
  (defvar default 'default))

;; Create faces for XEmacs
(unless (boundp 'font-lock-keyword-face)
  (copy-face 'bold 'font-lock-keyword-face))
(unless (boundp 'font-lock-constant-face)
  (copy-face 'font-lock-keyword-face 'font-lock-constant-face))

(provide 'php-mode)

;;; php-mode.el ends here
